@@ -4,6 +4,7 @@ import time
 from rich.live import Live
 
 from dstack._internal.cli.commands import APIBaseCommand
+from dstack._internal.cli.services.completion import GatewayNameCompleter
 from dstack._internal.cli.utils.common import (
     LIVE_TABLE_PROVISION_INTERVAL_SECS,
     LIVE_TABLE_REFRESH_RATE_PER_SEC,
@@ -12,6 +13,10 @@ from dstack._internal.cli.utils.common import (
 )
 from dstack._internal.cli.utils.gateway import get_gateways_table, print_gateways_table
 from dstack._internal.core.models.backends.base import BackendType
+from dstack._internal.core.models.gateways import GatewayConfiguration
+from dstack._internal.utils.logging import get_logger
+
+logger = get_logger(__name__)
 
 
 class GatewayCommand(APIBaseCommand):
@@ -40,7 +45,9 @@ class GatewayCommand(APIBaseCommand):
             )
 
         create_parser = subparsers.add_parser(
-            "create", help="Add a gateway", formatter_class=self._parser.formatter_class
+            "create",
+            help="Add a gateway. Deprecated in favor of `dstack apply` with gateway configuration.",
+            formatter_class=self._parser.formatter_class,
         )
         create_parser.set_defaults(subfunc=self._create)
         create_parser.add_argument(
@@ -59,7 +66,9 @@ class GatewayCommand(APIBaseCommand):
             "delete", help="Delete a gateway", formatter_class=self._parser.formatter_class
         )
         delete_parser.set_defaults(subfunc=self._delete)
-        delete_parser.add_argument("name", help="The name of the gateway")
+        delete_parser.add_argument(
+            "name", help="The name of the gateway"
+        ).completer = GatewayNameCompleter()
         delete_parser.add_argument(
             "-y", "--yes", action="store_true", help="Don't ask for confirmation"
         )
@@ -68,7 +77,9 @@ class GatewayCommand(APIBaseCommand):
             "update", help="Update a gateway", formatter_class=self._parser.formatter_class
         )
         update_parser.set_defaults(subfunc=self._update)
-        update_parser.add_argument("name", help="The name of the gateway")
+        update_parser.add_argument(
+            "name", help="The name of the gateway"
+        ).completer = GatewayNameCompleter()
         update_parser.add_argument(
             "--set-default", action="store_true", help="Set it the default gateway for the project"
         )
@@ -95,10 +106,16 @@ class GatewayCommand(APIBaseCommand):
             pass
 
     def _create(self, args: argparse.Namespace):
+        logger.warning(
+            "`dstack gateway create` is deperecated in favor of `dstack apply` with gateway configurations."
+        )
         with console.status("Creating gateway..."):
-            gateway = self.api.client.gateways.create(
-                self.api.project, args.name, BackendType(args.backend), args.region
+            configuration = GatewayConfiguration(
+                name=args.name,
+                backend=BackendType(args.backend),
+                region=args.region,
             )
+            gateway = self.api.client.gateways.create(self.api.project, configuration)
             if args.set_default:
                 self.api.client.gateways.set_default(self.api.project, gateway.name)
             if args.domain:
