@@ -1,10 +1,15 @@
 import os
 from argparse import Namespace
 
-import uvicorn
-
-from dstack import version
+from dstack._internal import settings
 from dstack._internal.cli.commands import BaseCommand
+from dstack._internal.core.errors import CLIError
+
+UVICORN_INSTALLED = True
+try:
+    import uvicorn
+except ImportError:
+    UVICORN_INSTALLED = False
 
 
 class ServerCommand(BaseCommand):
@@ -51,6 +56,12 @@ class ServerCommand(BaseCommand):
     def _command(self, args: Namespace):
         super()._command(args)
 
+        if not UVICORN_INSTALLED:
+            raise CLIError(
+                "Failed to start dstack server due to missing server dependencies."
+                "\nInstall server dependencies with `pip install dstack[server]` or `pip install dstack[all]`."
+            )
+
         os.environ["DSTACK_SERVER_HOST"] = args.host
         os.environ["DSTACK_SERVER_PORT"] = str(args.port)
         os.environ["DSTACK_SERVER_LOG_LEVEL"] = args.log_level
@@ -62,11 +73,12 @@ class ServerCommand(BaseCommand):
             os.environ["DSTACK_SERVER_ADMIN_TOKEN"] = args.token
         uvicorn_log_level = os.getenv("DSTACK_SERVER_UVICORN_LOG_LEVEL", "ERROR").lower()
         reload_disabled = os.getenv("DSTACK_SERVER_RELOAD_DISABLED") is not None
+
         uvicorn.run(
             "dstack._internal.server.main:app",
             host=args.host,
             port=args.port,
-            reload=version.__version__ is None and not reload_disabled,
+            reload=settings.DSTACK_VERSION is None and not reload_disabled,
             log_level=uvicorn_log_level,
             workers=1,
         )
