@@ -4,13 +4,11 @@ from azure.core.exceptions import ClientAuthenticationError
 from azure.identity import ClientSecretCredential, DefaultAzureCredential
 from azure.mgmt.subscription import SubscriptionClient
 
-from dstack._internal.core.errors import BackendAuthError
-from dstack._internal.core.models.backends.azure import (
+from dstack._internal.core.backends.azure.models import (
     AnyAzureCreds,
     AzureClientCreds,
-    AzureDefaultCreds,
 )
-from dstack._internal.core.models.common import is_core_model_instance
+from dstack._internal.core.errors import BackendAuthError
 
 AzureCredential = Union[ClientSecretCredential, DefaultAzureCredential]
 
@@ -22,7 +20,7 @@ def authenticate(creds: AnyAzureCreds) -> Tuple[AzureCredential, str]:
 
 
 def get_credential(creds: AnyAzureCreds) -> Tuple[AzureCredential, str]:
-    if is_core_model_instance(creds, AzureClientCreds):
+    if isinstance(creds, AzureClientCreds):
         credential = ClientSecretCredential(
             tenant_id=creds.tenant_id,
             client_id=creds.client_id,
@@ -39,11 +37,3 @@ def check_credential(credential: AzureCredential):
         list(client.subscriptions.list())
     except ClientAuthenticationError:
         raise BackendAuthError()
-
-
-def default_creds_available() -> bool:
-    try:
-        authenticate(AzureDefaultCreds())
-    except BackendAuthError:
-        return False
-    return True
