@@ -9,16 +9,22 @@ from kubernetes import client
 
 from dstack._internal.core.backends.base.compute import (
     Compute,
+    ComputeWithGatewaySupport,
+    generate_unique_gateway_instance_name,
+    generate_unique_instance_name_for_job,
     get_docker_commands,
     get_dstack_gateway_commands,
-    get_instance_name,
 )
 from dstack._internal.core.backends.base.offers import match_requirements
-from dstack._internal.core.backends.kubernetes.config import KubernetesConfig
+from dstack._internal.core.backends.kubernetes.models import (
+    KubernetesConfig,
+    KubernetesNetworkingConfig,
+)
 from dstack._internal.core.backends.kubernetes.utils import (
     get_api_from_config_data,
     get_cluster_public_ip,
 )
+from dstack._internal.core.consts import DSTACK_RUNNER_SSH_PORT
 from dstack._internal.core.errors import ComputeError
 from dstack._internal.core.models.backends.base import BackendType
 
@@ -44,7 +50,6 @@ from dstack._internal.utils.logging import get_logger
 
 logger = get_logger(__name__)
 
-RUNNER_SSH_PORT = 10022
 JUMP_POD_SSH_PORT = 22
 DEFAULT_NAMESPACE = "default"
 
@@ -52,9 +57,17 @@ NVIDIA_GPU_NAME_TO_GPU_INFO = {gpu.name: gpu for gpu in KNOWN_NVIDIA_GPUS}
 NVIDIA_GPU_NAMES = NVIDIA_GPU_NAME_TO_GPU_INFO.keys()
 
 
-class KubernetesCompute(Compute):
+class KubernetesCompute(
+    ComputeWithGatewaySupport,
+    Compute,
+):
     def __init__(self, config: KubernetesConfig):
-        self.config = config
+        super().__init__()
+        self.config = config.copy()
+        networking_config = self.config.networking
+        if networking_config is None:
+            networking_config = KubernetesNetworkingConfig()
+        self.networking_config = networking_config
         self.api = get_api_from_config_data(config.kubeconfig.data)
 
     def get_offers(
@@ -98,7 +111,7 @@ class KubernetesCompute(Compute):
         project_ssh_private_key: str,
         volumes: List[Volume],
     ) -> JobProvisioningData:
-        instance_name = get_instance_name(run, job)
+        instance_name = generate_unique_instance_name_for_job(run, job)
         commands = get_docker_commands(
             [run.run_spec.ssh_key_pub.strip(), project_ssh_public_key.strip()]
         )
@@ -107,7 +120,7 @@ class KubernetesCompute(Compute):
         # as an ssh proxy jump to connect to all other services in Kubernetes.
         # Setup jump pod in a separate thread to avoid long-running run_job.
         # In case the thread fails, the job will be failed and resubmitted.
-        jump_pod_hostname = self.config.networking.ssh_host
+        jump_pod_hostname = self.networking_config.ssh_host
         if jump_pod_hostname is None:
             jump_pod_hostname = get_cluster_public_ip(self.api)
             if jump_pod_hostname is None:
@@ -119,7 +132,7 @@ class KubernetesCompute(Compute):
             api=self.api,
             project_name=run.project_name,
             ssh_public_keys=[project_ssh_public_key.strip(), run.run_spec.ssh_key_pub.strip()],
-            jump_pod_port=self.config.networking.ssh_port,
+            jump_pod_port=self.networking_config.ssh_port,
         )
         if not created:
             threading.Thread(
@@ -149,7 +162,7 @@ class KubernetesCompute(Compute):
                             args=["-c", " && ".join(commands)],
                             ports=[
                                 client.V1ContainerPort(
-                                    container_port=RUNNER_SSH_PORT,
+                                    container_port=DSTACK_RUNNER_SSH_PORT,
                                 )
                             ],
                             security_context=client.V1SecurityContext(
@@ -173,7 +186,7 @@ class KubernetesCompute(Compute):
                 spec=client.V1ServiceSpec(
                     type="ClusterIP",
                     selector={"app.kubernetes.io/name": instance_name},
-                    ports=[client.V1ServicePort(port=RUNNER_SSH_PORT)],
+                    ports=[client.V1ServicePort(port=DSTACK_RUNNER_SSH_PORT)],
                 ),
             ),
         )
@@ -187,7 +200,7 @@ class KubernetesCompute(Compute):
             region="local",
             price=instance_offer.price,
             username="root",
-            ssh_port=RUNNER_SSH_PORT,
+            ssh_port=DSTACK_RUNNER_SSH_PORT,
             dockerized=False,
             ssh_proxy=SSHConnectionParams(
                 hostname=jump_pod_hostname,
@@ -230,7 +243,7 @@ class KubernetesCompute(Compute):
         # TODO: By default EKS creates a Classic Load Balancer for Load Balancer services.
         # Consider deploying an NLB. It seems it requires some extra configuration on the cluster:
         # https://docs.aws.amazon.com/eks/latest/userguide/network-load-balancing.html
-        instance_name = configuration.instance_name
+        instance_name = generate_unique_gateway_instance_name(configuration)
         commands = _get_gateway_commands(authorized_keys=[configuration.ssh_key_pub])
         self.api.create_namespaced_pod(
             namespace=DEFAULT_NAMESPACE,
