@@ -1,6 +1,11 @@
 from typing import List, Optional
 
-from dstack._internal.core.backends.base.compute import Compute
+from dstack._internal.core.backends.base.compute import (
+    Compute,
+    ComputeWithCreateInstanceSupport,
+    ComputeWithVolumeSupport,
+)
+from dstack._internal.core.consts import DSTACK_RUNNER_SSH_PORT
 from dstack._internal.core.models.backends.base import BackendType
 from dstack._internal.core.models.instances import (
     InstanceAvailability,
@@ -10,6 +15,7 @@ from dstack._internal.core.models.instances import (
     InstanceType,
     Resources,
 )
+from dstack._internal.core.models.placement import PlacementGroup
 from dstack._internal.core.models.runs import Job, JobProvisioningData, Requirements, Run
 from dstack._internal.core.models.volumes import Volume, VolumeProvisioningData
 from dstack._internal.utils.logging import get_logger
@@ -17,7 +23,11 @@ from dstack._internal.utils.logging import get_logger
 logger = get_logger(__name__)
 
 
-class LocalCompute(Compute):
+class LocalCompute(
+    ComputeWithCreateInstanceSupport,
+    ComputeWithVolumeSupport,
+    Compute,
+):
     def get_offers(
         self, requirements: Optional[Requirements] = None
     ) -> List[InstanceOfferWithAvailability]:
@@ -44,6 +54,7 @@ class LocalCompute(Compute):
         self,
         instance_offer: InstanceOfferWithAvailability,
         instance_config: InstanceConfiguration,
+        placement_group: Optional[PlacementGroup],
     ) -> JobProvisioningData:
         return JobProvisioningData(
             backend=instance_offer.backend,
@@ -54,7 +65,7 @@ class LocalCompute(Compute):
             region="",
             price=instance_offer.price,
             username="root",
-            ssh_port=10022,
+            ssh_port=DSTACK_RUNNER_SSH_PORT,
             ssh_proxy=None,
             dockerized=True,
             backend_data=None,
@@ -78,12 +89,18 @@ class LocalCompute(Compute):
             region="",
             price=instance_offer.price,
             username="root",
-            ssh_port=10022,
+            ssh_port=DSTACK_RUNNER_SSH_PORT,
             ssh_proxy=None,
             dockerized=True,
             backend_data=None,
         )
 
+    def register_volume(self, volume: Volume) -> VolumeProvisioningData:
+        return VolumeProvisioningData(
+            volume_id=volume.volume_id,
+            size_gb=volume.configuration.size_gb,
+        )
+
     def create_volume(self, volume: Volume) -> VolumeProvisioningData:
         return VolumeProvisioningData(
             volume_id=volume.name,
@@ -93,8 +110,10 @@ class LocalCompute(Compute):
     def delete_volume(self, volume: Volume):
         pass
 
-    def attach_volume(self, volume: Volume, instance_id: str):
+    def attach_volume(self, volume: Volume, provisioning_data: JobProvisioningData):
         pass
 
-    def detach_volume(self, volume: Volume, instance_id: str):
+    def detach_volume(
+        self, volume: Volume, provisioning_data: JobProvisioningData, force: bool = False
+    ):
         pass
