@@ -2,12 +2,11 @@ import oci
 from typing_extensions import Any, Mapping
 
 from dstack._internal.core.backends.oci.exceptions import any_oci_exception
-from dstack._internal.core.models.backends.oci import AnyOCICreds, OCIDefaultCreds
-from dstack._internal.core.models.common import is_core_model_instance
+from dstack._internal.core.backends.oci.models import AnyOCICreds, OCIDefaultCreds
 
 
 def get_client_config(creds: AnyOCICreds) -> Mapping[str, Any]:
-    if is_core_model_instance(creds, OCIDefaultCreds):
+    if isinstance(creds, OCIDefaultCreds):
         return oci.config.from_file(file_location=creds.file, profile_name=creds.profile)
     return creds.dict(exclude={"type"})
 
@@ -20,7 +19,3 @@ def creds_valid(creds: AnyOCICreds) -> bool:
     except any_oci_exception:
         return False
     return True
-
-
-def default_creds_available() -> bool:
-    return creds_valid(OCIDefaultCreds())
