@@ -3,11 +3,15 @@ from typing import List, Optional
 import gpuhunt
 from gpuhunt.providers.vastai import VastAIProvider
 
-from dstack._internal.core.backends.base import Compute
-from dstack._internal.core.backends.base.compute import get_docker_commands, get_instance_name
+from dstack._internal.core.backends.base.backend import Compute
+from dstack._internal.core.backends.base.compute import (
+    generate_unique_instance_name_for_job,
+    get_docker_commands,
+)
 from dstack._internal.core.backends.base.offers import get_catalog_offers
 from dstack._internal.core.backends.vastai.api_client import VastAIAPIClient
-from dstack._internal.core.backends.vastai.config import VastAIConfig
+from dstack._internal.core.backends.vastai.models import VastAIConfig
+from dstack._internal.core.consts import DSTACK_RUNNER_SSH_PORT
 from dstack._internal.core.errors import ProvisioningError
 from dstack._internal.core.models.backends.base import BackendType
 from dstack._internal.core.models.instances import (
@@ -22,8 +26,13 @@ from dstack._internal.utils.logging import get_logger
 logger = get_logger(__name__)
 
 
+# Undocumented but names of len 60 work
+MAX_INSTANCE_NAME_LEN = 60
+
+
 class VastAICompute(Compute):
     def __init__(self, config: VastAIConfig):
+        super().__init__()
         self.config = config
         self.api_client = VastAIAPIClient(config.creds.api_key)
         self.catalog = gpuhunt.Catalog(balance_resources=False, auto_reload=False)
@@ -34,7 +43,8 @@ class VastAICompute(Compute):
                     "reliability2": {"gte": 0.9},
                     "inet_down": {"gt": 128},
                     "verified": {"eq": True},
-                    "cuda_max_good": {"gte": 11.8},
+                    "cuda_max_good": {"gte": 12.1},
+                    "compute_cap": {"gte": 600},
                 }
             )
         )
@@ -68,11 +78,14 @@ class VastAICompute(Compute):
         project_ssh_private_key: str,
         volumes: List[Volume],
     ) -> JobProvisioningData:
+        instance_name = generate_unique_instance_name_for_job(
+            run, job, max_length=MAX_INSTANCE_NAME_LEN
+        )
         commands = get_docker_commands(
             [run.run_spec.ssh_key_pub.strip(), project_ssh_public_key.strip()]
         )
         resp = self.api_client.create_instance(
-            instance_name=get_instance_name(run, job),
+            instance_name=instance_name,
             bundle_id=instance_offer.instance.name,
             image_name=job.job_spec.image_name,
             onstart=" && ".join(commands),
@@ -110,7 +123,9 @@ class VastAICompute(Compute):
         if resp is not None:
             if resp["actual_status"] == "running":
                 provisioning_data.hostname = resp["public_ipaddr"].strip()
-                provisioning_data.ssh_port = int(resp["ports"]["10022/tcp"][0]["HostPort"])
+                provisioning_data.ssh_port = int(
+                    resp["ports"][f"{DSTACK_RUNNER_SSH_PORT}/tcp"][0]["HostPort"]
+                )
             if (
                 resp["actual_status"] == "created"
                 and ": OCI runtime create failed:" in resp["status_msg"]
