@@ -1,7 +1,4 @@
 import enum
-from typing import List, Optional
-
-from dstack._internal.core.models.common import CoreModel
 
 
 class BackendType(str, enum.Enum):
@@ -9,23 +6,30 @@ class BackendType(str, enum.Enum):
     Attributes:
         AWS (BackendType): Amazon Web Services
         AZURE (BackendType): Microsoft Azure
+        CLOUDRIFT (BackendType): CloudRift
         CUDO (BackendType): Cudo
         DSTACK (BackendType): dstack Sky
         GCP (BackendType): Google Cloud Platform
         DATACRUNCH (BackendType): DataCrunch
+        HOTAISLE (BackendType): Hot Aisle
         KUBERNETES (BackendType): Kubernetes
         LAMBDA (BackendType): Lambda Cloud
+        NEBIUS (BackendType): Nebius AI Cloud
+        OCI (BackendType): Oracle Cloud Infrastructure
         RUNPOD (BackendType): Runpod Cloud
         TENSORDOCK (BackendType): TensorDock Marketplace
         VASTAI (BackendType): Vast.ai Marketplace
+        VULTR (BackendType): Vultr
     """
 
     AWS = "aws"
     AZURE = "azure"
+    CLOUDRIFT = "cloudrift"
     CUDO = "cudo"
     DATACRUNCH = "datacrunch"
     DSTACK = "dstack"
     GCP = "gcp"
+    HOTAISLE = "hotaisle"
     KUBERNETES = "kubernetes"
     LAMBDA = "lambda"
     LOCAL = "local"
@@ -35,18 +39,4 @@ class BackendType(str, enum.Enum):
     RUNPOD = "runpod"
     TENSORDOCK = "tensordock"
     VASTAI = "vastai"
-
-
-class ConfigElementValue(CoreModel):
-    value: str
-    label: str
-
-
-class ConfigElement(CoreModel):
-    selected: Optional[str] = None
-    values: List[ConfigElementValue] = []
-
-
-class ConfigMultiElement(CoreModel):
-    selected: List[str] = []
-    values: List[ConfigElementValue] = []
+    VULTR = "vultr"
