@@ -1,10 +1,20 @@
 import re
 from enum import Enum
-from typing import Any, Type, TypeVar, Union
+from typing import Any, Callable, Optional, Union
 
+import orjson
 from pydantic import Field
 from pydantic_duality import DualBaseModel
-from typing_extensions import Annotated, TypeGuard
+from typing_extensions import Annotated
+
+from dstack._internal.utils.json_utils import pydantic_orjson_dumps
+
+IncludeExcludeFieldType = Union[int, str]
+IncludeExcludeSetType = set[IncludeExcludeFieldType]
+IncludeExcludeDictType = dict[
+    IncludeExcludeFieldType, Union[bool, IncludeExcludeSetType, "IncludeExcludeDictType"]
+]
+IncludeExcludeType = Union[IncludeExcludeSetType, IncludeExcludeDictType]
 
 
 # DualBaseModel creates two classes for the model:
@@ -13,7 +23,40 @@ from typing_extensions import Annotated, TypeGuard
 # This allows to use the same model both for a strict parsing of the user input and
 # for a permissive parsing of the server responses.
 class CoreModel(DualBaseModel):
-    pass
+    class Config:
+        json_loads = orjson.loads
+        json_dumps = pydantic_orjson_dumps
+
+    def json(
+        self,
+        *,
+        include: Optional[IncludeExcludeType] = None,
+        exclude: Optional[IncludeExcludeType] = None,
+        by_alias: bool = False,
+        skip_defaults: Optional[bool] = None,  # ignore as it's deprecated
+        exclude_unset: bool = False,
+        exclude_defaults: bool = False,
+        exclude_none: bool = False,
+        encoder: Optional[Callable[[Any], Any]] = None,
+        models_as_dict: bool = True,  # does not seems to be needed by dstack or dependencies
+        **dumps_kwargs: Any,
+    ) -> str:
+        """
+        Override `json()` method so that it calls `dict()`.
+        Allows changing how models are serialized by overriding `dict()` only.
+        By default, `json()` won't call `dict()`, so changes applied in `dict()` won't take place.
+        """
+        data = self.dict(
+            by_alias=by_alias,
+            include=include,
+            exclude=exclude,
+            exclude_unset=exclude_unset,
+            exclude_defaults=exclude_defaults,
+            exclude_none=exclude_none,
+        )
+        if self.__custom_root_type__:
+            data = data["__root__"]
+        return self.__config__.json_dumps(data, default=encoder, **dumps_kwargs)
 
 
 class Duration(int):
@@ -71,13 +114,6 @@ class ApplyAction(str, Enum):
     UPDATE = "update"  # resource is to be updated in-place
 
 
-_CM = TypeVar("_CM", bound=CoreModel)
-
-
-def is_core_model_instance(instance: Any, class_: Type[_CM]) -> TypeGuard[_CM]:
-    """
-    Implements isinstance check for CoreModel such that
-    models parsed with MyModel.__response__ pass the check against MyModel.
-    See https://github.com/dstackai/dstack/issues/1124
-    """
-    return isinstance(instance, class_) or isinstance(instance, class_.__response__)
+class NetworkMode(str, Enum):
+    HOST = "host"
+    BRIDGE = "bridge"
