@@ -3,7 +3,7 @@ from typing import List, Optional
 
 from pydantic import UUID4
 
-from dstack._internal.core.models.backends import BackendInfo
+from dstack._internal.core.backends.models import BackendInfo
 from dstack._internal.core.models.common import CoreModel
 from dstack._internal.core.models.users import ProjectRole, User
 
@@ -25,3 +25,4 @@ class Project(CoreModel):
     created_at: Optional[datetime] = None
     backends: List[BackendInfo]
     members: List[Member]
+    is_public: bool = False
