@@ -1,13 +1,15 @@
 import math
+from collections.abc import Mapping
 from typing import Any, Dict, Generic, List, Optional, Tuple, TypeVar, Union
 
 import gpuhunt
-from pydantic import Field, root_validator, validator
+from pydantic import Field, parse_obj_as, root_validator, validator
 from pydantic.generics import GenericModel
 from typing_extensions import Annotated
 
 from dstack._internal.core.models.common import CoreModel
 from dstack._internal.utils.common import pretty_resources
+from dstack._internal.utils.json_schema import add_extra_schema_types
 from dstack._internal.utils.logging import get_logger
 
 logger = get_logger(__name__)
@@ -124,14 +126,91 @@ class ComputeCapability(Tuple[int, int]):
 
 DEFAULT_CPU_COUNT = Range[int](min=2)
 DEFAULT_MEMORY_SIZE = Range[Memory](min=Memory.parse("8GB"))
-DEFAULT_GPU_COUNT = Range[int](min=1, max=1)
+DEFAULT_GPU_COUNT = Range[int](min=1)
+
+
+class CPUSpec(CoreModel):
+    class Config:
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any]):
+            add_extra_schema_types(
+                schema["properties"]["count"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+
+    arch: Annotated[
+        Optional[gpuhunt.CPUArchitecture],
+        Field(description="The CPU architecture, one of: `x86`, `arm`"),
+    ] = None
+    count: Annotated[Range[int], Field(description="The number of CPU cores")] = DEFAULT_CPU_COUNT
+
+    @classmethod
+    def __get_validators__(cls):
+        yield cls.parse
+        yield cls.validate
+
+    @classmethod
+    def parse(cls, v: Any) -> Any:
+        if isinstance(v, int):
+            v = str(v)
+        if isinstance(v, str):
+            tokens = v.replace(" ", "").split(":")
+            spec = {}
+            for token in tokens:
+                if not token:
+                    raise ValueError(f"CPU spec contains empty token: {v}")
+                if ".." in token or token.isdigit():
+                    if "count" in spec:
+                        raise ValueError(f"CPU spec count conflict: {v}")
+                    spec["count"] = token
+                else:
+                    try:
+                        arch = gpuhunt.CPUArchitecture.cast(token)
+                    except ValueError:
+                        raise ValueError(f"Invalid CPU architecture: {v}")
+                    if "arch" in spec:
+                        raise ValueError(f"CPU spec arch conflict: {v}")
+                    spec["arch"] = arch
+            return spec
+        # Range and min/max dict - for backward compatibility
+        if isinstance(v, Range):
+            return {"arch": None, "count": v}
+        if isinstance(v, Mapping) and v.keys() == {"min", "max"}:
+            return {"arch": None, "count": v}
+        return v
+
+    @validator("arch", pre=True)
+    def _validate_arch(cls, v: Any) -> Any:
+        if v is None:
+            return None
+        if isinstance(v, gpuhunt.CPUArchitecture):
+            return v
+        if isinstance(v, str):
+            return gpuhunt.CPUArchitecture.cast(v)
+        return v
 
 
 class GPUSpec(CoreModel):
+    class Config:
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any]):
+            add_extra_schema_types(
+                schema["properties"]["count"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+            add_extra_schema_types(
+                schema["properties"]["memory"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+            add_extra_schema_types(
+                schema["properties"]["total_memory"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+
     vendor: Annotated[
         Optional[gpuhunt.AcceleratorVendor],
         Field(
-            description="The vendor of the GPU/accelerator, one of: `nvidia`, `amd`, `google` (alias: `tpu`)"
+            description="The vendor of the GPU/accelerator, one of: `nvidia`, `amd`, `google` (alias: `tpu`), `intel`"
         ),
     ] = None
     name: Annotated[
@@ -229,10 +308,20 @@ class GPUSpec(CoreModel):
         v = v.lower()
         if v == "tpu":
             return gpuhunt.AcceleratorVendor.GOOGLE
+        if v == "tt":
+            return gpuhunt.AcceleratorVendor.TENSTORRENT
         return gpuhunt.AcceleratorVendor.cast(v)
 
 
 class DiskSpec(CoreModel):
+    class Config:
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any]):
+            add_extra_schema_types(
+                schema["properties"]["size"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+
     size: Annotated[Range[Memory], Field(description="Disk size")]
 
     @classmethod
@@ -254,13 +343,31 @@ class ResourcesSpec(CoreModel):
     class Config:
         @staticmethod
         def schema_extra(schema: Dict[str, Any]):
-            schema.clear()
-            # replace strict schema with a more permissive one
-            ref_template = "#/definitions/ResourcesSpecRequest/definitions/{model}"
-            for field, value in ResourcesSpecSchema.schema(ref_template=ref_template).items():
-                schema[field] = value
+            add_extra_schema_types(
+                schema["properties"]["cpu"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+            add_extra_schema_types(
+                schema["properties"]["memory"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+            add_extra_schema_types(
+                schema["properties"]["shm_size"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+            add_extra_schema_types(
+                schema["properties"]["gpu"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
+            add_extra_schema_types(
+                schema["properties"]["disk"],
+                extra_types=[{"type": "integer"}, {"type": "string"}],
+            )
 
-    cpu: Annotated[Range[int], Field(description="The number of CPU cores")] = DEFAULT_CPU_COUNT
+    # TODO: Remove Range[int] in 0.20. Range[int] for backward compatibility only.
+    cpu: Annotated[Union[CPUSpec, Range[int]], Field(description="The CPU requirements")] = (
+        CPUSpec()
+    )
     memory: Annotated[Range[Memory], Field(description="The RAM size (e.g., `8GB`)")] = (
         DEFAULT_MEMORY_SIZE
     )
@@ -276,7 +383,9 @@ class ResourcesSpec(CoreModel):
     disk: Annotated[Optional[DiskSpec], Field(description="The disk resources")] = DEFAULT_DISK
 
     def pretty_format(self) -> str:
-        resources: Dict[str, Any] = dict(cpus=self.cpu, memory=self.memory)
+        # TODO: Remove in 0.20. Use self.cpu directly
+        cpu = parse_obj_as(CPUSpec, self.cpu)
+        resources: Dict[str, Any] = dict(cpu_arch=cpu.arch, cpus=cpu.count, memory=self.memory)
         if self.gpu:
             gpu = self.gpu
             resources.update(
@@ -291,73 +400,17 @@ class ResourcesSpec(CoreModel):
         res = pretty_resources(**resources)
         return res
 
+    def dict(self, *args, **kwargs) -> Dict:
+        # super() does not work with pydantic-duality
+        res = CoreModel.dict(self, *args, **kwargs)
+        self._update_serialized_cpu(res)
+        return res
 
-IntRangeLike = Union[Range[Union[int, str]], int, str]
-MemoryRangeLike = Union[Range[Union[Memory, float, int, str]], float, int, str]
-MemoryLike = Union[Memory, float, int, str]
-GPULike = Union[GPUSpec, "GPUSpecSchema", int, str]
-DiskLike = Union[DiskSpec, "DiskSpecSchema", float, int, str]
-ComputeCapabilityLike = Union[ComputeCapability, float, str]
-
-
-class GPUSpecSchema(CoreModel):
-    vendor: Annotated[
-        Optional[gpuhunt.AcceleratorVendor],
-        Field(
-            description="The vendor of the GPU/accelerator, one of: `nvidia`, `amd`, `google` (alias: `tpu`)"
-        ),
-    ] = None
-    name: Annotated[
-        Optional[Union[List[str], str]], Field(description="The GPU name or list of names")
-    ] = None
-    count: Annotated[IntRangeLike, Field(description="The number of GPUs")] = DEFAULT_GPU_COUNT
-    memory: Annotated[
-        Optional[MemoryRangeLike],
-        Field(
-            description="The RAM size (e.g., `16GB`). Can be set to a range (e.g. `16GB..`, or `16GB..80GB`)"
-        ),
-    ] = None
-    total_memory: Annotated[
-        Optional[MemoryRangeLike],
-        Field(
-            description="The total RAM size (e.g., `32GB`). Can be set to a range (e.g. `16GB..`, or `16GB..80GB`)"
-        ),
-    ] = None
-    compute_capability: Annotated[
-        Optional[ComputeCapabilityLike],
-        Field(description="The minimum compute capability of the GPU (e.g., `7.5`)"),
-    ] = None
-
-
-class DiskSpecSchema(CoreModel):
-    size: Annotated[
-        MemoryRangeLike,
-        Field(
-            description="The disk size. Can be a string (e.g., `100GB` or `100GB..`) or an object"
-        ),
-    ]
-
-
-class ResourcesSpecSchema(CoreModel):
-    cpu: Annotated[Optional[IntRangeLike], Field(description="The number of CPU cores")] = (
-        DEFAULT_CPU_COUNT
-    )
-    memory: Annotated[
-        Optional[MemoryRangeLike],
-        Field(description="The RAM size (e.g., `8GB`)"),
-    ] = DEFAULT_MEMORY_SIZE
-    shm_size: Annotated[
-        Optional[MemoryLike],
-        Field(
-            description="The size of shared memory (e.g., `8GB`). "
-            "If you are using parallel communicating processes (e.g., dataloaders in PyTorch), "
-            "you may need to configure this"
-        ),
-    ] = None
-    gpu: Annotated[
-        Optional[GPULike],
-        Field(
-            description="The GPU requirements. Can be set to a number, a string (e.g. `A100`, `80GB:2`, etc.), or an object"
-        ),
-    ] = None
-    disk: Annotated[Optional[DiskLike], Field(description="The disk resources")] = DEFAULT_DISK
+    # TODO: Remove in 0.20. Added for backward compatibility.
+    def _update_serialized_cpu(self, values: Dict):
+        cpu = values["cpu"]
+        if cpu:
+            arch = cpu.get("arch")
+            count = cpu.get("count")
+            if count and arch in [None, gpuhunt.CPUArchitecture.X86.value]:
+                values["cpu"] = count
