@@ -2,7 +2,7 @@ import atexit
 import re
 import time
 from pathlib import Path
-from typing import Optional
+from typing import Optional, Union
 
 import psutil
 
@@ -14,6 +14,8 @@ from dstack._internal.core.services.ssh.ports import PortsLock
 from dstack._internal.core.services.ssh.tunnel import SSHTunnel, ports_to_forwarded_sockets
 from dstack._internal.utils.path import FilePath, PathLike
 from dstack._internal.utils.ssh import (
+    default_ssh_config_path,
+    get_host_config,
     include_ssh_config,
     normalize_path,
     update_ssh_config,
@@ -26,7 +28,7 @@ _SSH_TUNNEL_REGEX = re.compile(r"(?:[\w.-]+:)?(?P<local_port>\d+):localhost:(?P<
 class SSHAttach:
     @classmethod
     def get_control_sock_path(cls, run_name: str) -> Path:
-        return ConfigManager().dstack_ssh_dir / f"{run_name}.control.sock"
+        return ConfigManager().dstack_ssh_dir / f"%r@{run_name}.control.sock"
 
     @classmethod
     def reuse_ports_lock(cls, run_name: str) -> Optional[PortsLock]:
@@ -54,12 +56,15 @@ class SSHAttach:
         self,
         hostname: str,
         ssh_port: int,
+        container_ssh_port: int,
         user: str,
+        container_user: str,
         id_rsa_path: PathLike,
         ports_lock: PortsLock,
         run_name: str,
         dockerized: bool,
         ssh_proxy: Optional[SSHConnectionParams] = None,
+        service_port: Optional[int] = None,
         local_backend: bool = False,
         bind_address: Optional[str] = None,
     ):
@@ -73,7 +78,7 @@ class SSHAttach:
         self.control_sock_path = FilePath(control_sock_path)
         self.identity_file = FilePath(id_rsa_path)
         self.tunnel = SSHTunnel(
-            destination=run_name,
+            destination=f"root@{run_name}",
             identity=self.identity_file,
             forwarded_sockets=ports_to_forwarded_sockets(
                 ports=self.ports,
@@ -86,52 +91,108 @@ class SSHAttach:
             },
         )
         self.ssh_proxy = ssh_proxy
-        if ssh_proxy is None:
-            self.host_config = {
+        self.service_port = service_port
+
+        hosts: dict[str, dict[str, Union[str, int, FilePath]]] = {}
+        self.hosts = hosts
+
+        if local_backend:
+            hosts[run_name] = {
                 "HostName": hostname,
-                "Port": ssh_port,
-                "User": user,
+                "Port": container_ssh_port,
+                "User": container_user,
                 "IdentityFile": self.identity_file,
                 "IdentitiesOnly": "yes",
                 "StrictHostKeyChecking": "no",
                 "UserKnownHostsFile": "/dev/null",
             }
-        else:
-            self.host_config = {
-                "HostName": ssh_proxy.hostname,
-                "Port": ssh_proxy.port,
-                "User": ssh_proxy.username,
-                "IdentityFile": self.identity_file,
-                "IdentitiesOnly": "yes",
-                "StrictHostKeyChecking": "no",
-                "UserKnownHostsFile": "/dev/null",
-            }
-        if dockerized and not local_backend:
-            self.container_config = {
+        elif dockerized:
+            if ssh_proxy is not None:
+                # SSH instance with jump host
+                # dstack has no IdentityFile for jump host, it must be either preconfigured
+                # in the ~/.ssh/config or loaded into ssh-agent
+                hosts[f"{run_name}-jump-host"] = {
+                    "HostName": ssh_proxy.hostname,
+                    "Port": ssh_proxy.port,
+                    "User": ssh_proxy.username,
+                    "StrictHostKeyChecking": "no",
+                    "UserKnownHostsFile": "/dev/null",
+                }
+                jump_host_config = get_host_config(ssh_proxy.hostname, default_ssh_config_path)
+                jump_host_identity_files = jump_host_config.get("identityfile")
+                if jump_host_identity_files:
+                    hosts[f"{run_name}-jump-host"].update(
+                        {
+                            "IdentityFile": jump_host_identity_files[0],
+                            "IdentitiesOnly": "yes",
+                        }
+                    )
+                hosts[f"{run_name}-host"] = {
+                    "HostName": hostname,
+                    "Port": ssh_port,
+                    "User": user,
+                    "IdentityFile": self.identity_file,
+                    "IdentitiesOnly": "yes",
+                    "StrictHostKeyChecking": "no",
+                    "UserKnownHostsFile": "/dev/null",
+                    "ProxyJump": f"{run_name}-jump-host",
+                }
+            else:
+                # Regular SSH instance or VM-based cloud instance
+                hosts[f"{run_name}-host"] = {
+                    "HostName": hostname,
+                    "Port": ssh_port,
+                    "User": user,
+                    "IdentityFile": self.identity_file,
+                    "IdentitiesOnly": "yes",
+                    "StrictHostKeyChecking": "no",
+                    "UserKnownHostsFile": "/dev/null",
+                }
+            hosts[run_name] = {
                 "HostName": "localhost",
-                "Port": 10022,
-                "User": "root",  # TODO(#1535): support non-root images properly
+                "Port": container_ssh_port,
+                "User": container_user,
                 "IdentityFile": self.identity_file,
                 "IdentitiesOnly": "yes",
                 "StrictHostKeyChecking": "no",
                 "UserKnownHostsFile": "/dev/null",
                 "ProxyJump": f"{run_name}-host",
             }
-        elif ssh_proxy is not None:
-            self.container_config = {
-                "HostName": hostname,
-                "Port": ssh_port,
-                "User": user,
-                "IdentityFile": self.identity_file,
-                "IdentitiesOnly": "yes",
-                "StrictHostKeyChecking": "no",
-                "UserKnownHostsFile": "/dev/null",
-                "ProxyJump": f"{run_name}-jump-host",
-            }
         else:
-            self.container_config = None
-        if self.container_config is not None and get_ssh_client_info().supports_multiplexing:
-            self.container_config.update(
+            if ssh_proxy is not None:
+                # Kubernetes
+                hosts[f"{run_name}-jump-host"] = {
+                    "HostName": ssh_proxy.hostname,
+                    "Port": ssh_proxy.port,
+                    "User": ssh_proxy.username,
+                    "IdentityFile": self.identity_file,
+                    "IdentitiesOnly": "yes",
+                    "StrictHostKeyChecking": "no",
+                    "UserKnownHostsFile": "/dev/null",
+                }
+                hosts[run_name] = {
+                    "HostName": hostname,
+                    "Port": ssh_port,
+                    "User": container_user,
+                    "IdentityFile": self.identity_file,
+                    "IdentitiesOnly": "yes",
+                    "StrictHostKeyChecking": "no",
+                    "UserKnownHostsFile": "/dev/null",
+                    "ProxyJump": f"{run_name}-jump-host",
+                }
+            else:
+                # Container-based backends
+                hosts[run_name] = {
+                    "HostName": hostname,
+                    "Port": ssh_port,
+                    "User": container_user,
+                    "IdentityFile": self.identity_file,
+                    "IdentitiesOnly": "yes",
+                    "StrictHostKeyChecking": "no",
+                    "UserKnownHostsFile": "/dev/null",
+                }
+        if get_ssh_client_info().supports_multiplexing:
+            hosts[run_name].update(
                 {
                     "ControlMaster": "auto",
                     "ControlPath": self.control_sock_path,
@@ -140,14 +201,8 @@ class SSHAttach:
 
     def attach(self):
         include_ssh_config(self.ssh_config_path)
-        if self.container_config is None:
-            update_ssh_config(self.ssh_config_path, self.run_name, self.host_config)
-        elif self.ssh_proxy is not None:
-            update_ssh_config(self.ssh_config_path, f"{self.run_name}-jump-host", self.host_config)
-            update_ssh_config(self.ssh_config_path, self.run_name, self.container_config)
-        else:
-            update_ssh_config(self.ssh_config_path, f"{self.run_name}-host", self.host_config)
-            update_ssh_config(self.ssh_config_path, self.run_name, self.container_config)
+        for host, options in self.hosts.items():
+            update_ssh_config(self.ssh_config_path, host, options)
 
         max_retries = 10
         self._ports_lock.release()
@@ -165,9 +220,8 @@ class SSHAttach:
 
     def detach(self):
         self.tunnel.close()
-        update_ssh_config(self.ssh_config_path, f"{self.run_name}-jump-host", {})
-        update_ssh_config(self.ssh_config_path, f"{self.run_name}-host", {})
-        update_ssh_config(self.ssh_config_path, self.run_name, {})
+        for host in self.hosts:
+            update_ssh_config(self.ssh_config_path, host, {})
 
     def __enter__(self):
         self.attach()
