@@ -9,44 +9,60 @@ import dstack._internal.server.services.gateways as gateways
 from dstack._internal.core.errors import ResourceNotExistsError
 from dstack._internal.server.db import get_session
 from dstack._internal.server.models import ProjectModel, UserModel
-from dstack._internal.server.security.permissions import ProjectAdmin, ProjectMember
+from dstack._internal.server.security.permissions import (
+    ProjectAdmin,
+    ProjectMemberOrPublicAccess,
+)
+from dstack._internal.server.utils.routers import (
+    CustomORJSONResponse,
+    get_base_api_additional_responses,
+)
 
-router = APIRouter(prefix="/api/project/{project_name}/gateways", tags=["gateways"])
+router = APIRouter(
+    prefix="/api/project/{project_name}/gateways",
+    tags=["gateways"],
+    responses=get_base_api_additional_responses(),
+)
 
 
-@router.post("/list")
+@router.post("/list", response_model=List[models.Gateway])
 async def list_gateways(
     session: AsyncSession = Depends(get_session),
-    user_project: Tuple[UserModel, ProjectModel] = Depends(ProjectMember()),
-) -> List[models.Gateway]:
+    user_project: Tuple[UserModel, ProjectModel] = Depends(ProjectMemberOrPublicAccess()),
+):
     _, project = user_project
-    return await gateways.list_project_gateways(session=session, project=project)
+    return CustomORJSONResponse(
+        await gateways.list_project_gateways(session=session, project=project)
+    )
 
 
-@router.post("/get")
+@router.post("/get", response_model=models.Gateway)
 async def get_gateway(
     body: schemas.GetGatewayRequest,
     session: AsyncSession = Depends(get_session),
-    user_project: Tuple[UserModel, ProjectModel] = Depends(ProjectMember()),
-) -> models.Gateway:
+    user_project: Tuple[UserModel, ProjectModel] = Depends(ProjectMemberOrPublicAccess()),
+):
     _, project = user_project
     gateway = await gateways.get_gateway_by_name(session=session, project=project, name=body.name)
     if gateway is None:
         raise ResourceNotExistsError()
-    return gateway
+    return CustomORJSONResponse(gateway)
 
 
-@router.post("/create")
+@router.post("/create", response_model=models.Gateway)
 async def create_gateway(
     body: schemas.CreateGatewayRequest,
     session: AsyncSession = Depends(get_session),
     user_project: Tuple[UserModel, ProjectModel] = Depends(ProjectAdmin()),
-) -> models.Gateway:
-    _, project = user_project
-    return await gateways.create_gateway(
-        session=session,
-        project=project,
-        configuration=body.configuration,
+):
+    user, project = user_project
+    return CustomORJSONResponse(
+        await gateways.create_gateway(
+            session=session,
+            user=user,
+            project=project,
+            configuration=body.configuration,
+        )
     )
 
 
@@ -74,13 +90,15 @@ async def set_default_gateway(
     await gateways.set_default_gateway(session=session, project=project, name=body.name)
 
 
-@router.post("/set_wildcard_domain")
+@router.post("/set_wildcard_domain", response_model=models.Gateway)
 async def set_gateway_wildcard_domain(
     body: schemas.SetWildcardDomainRequest,
     session: AsyncSession = Depends(get_session),
     user_project: Tuple[UserModel, ProjectModel] = Depends(ProjectAdmin()),
-) -> models.Gateway:
+):
     _, project = user_project
-    return await gateways.set_gateway_wildcard_domain(
-        session=session, project=project, name=body.name, wildcard_domain=body.wildcard_domain
+    return CustomORJSONResponse(
+        await gateways.set_gateway_wildcard_domain(
+            session=session, project=project, name=body.name, wildcard_domain=body.wildcard_domain
+        )
     )
