@@ -43,9 +43,17 @@ class DockerImage(CoreModel):
 
 
 class ImageConfig(CoreModel):
+    user: Annotated[Optional[str], Field(alias="User")] = None
     entrypoint: Annotated[Optional[List[str]], Field(alias="Entrypoint")] = None
     cmd: Annotated[Optional[List[str]], Field(alias="Cmd")] = None
 
+    @validator("user")
+    def normalize_user(cls, v: Optional[str]) -> Optional[str]:
+        # If USER is not set, the corresponding field may be missing or set to an empty string
+        if v == "":
+            return None
+        return v
+
 
 class ImageConfigObject(CoreModel):
     config: ImageConfig = ImageConfig()
@@ -83,8 +91,7 @@ def get_image_config(image_name: str, registry_auth: Optional[RegistryAuth]) ->
             config_resp = join_byte_stream_checked(config_stream, MAX_CONFIG_OBJECT_SIZE)
             if config_resp is None:
                 raise DockerRegistryError(
-                    "Image config object exceeds the size limit of "
-                    f"{MAX_CONFIG_OBJECT_SIZE} bytes"
+                    f"Image config object exceeds the size limit of {MAX_CONFIG_OBJECT_SIZE} bytes"
                 )
             return ImageConfigObject.__response__.parse_raw(config_resp)
 
