@@ -1,26 +1,26 @@
 import asyncio
 import datetime
 import uuid
-from datetime import timedelta, timezone
+from datetime import timedelta
+from functools import partial
 from typing import List, Optional, Sequence
-from urllib.parse import urlparse
 
 import httpx
 from sqlalchemy import func, select, update
 from sqlalchemy.ext.asyncio import AsyncSession
-from sqlalchemy.orm import joinedload, selectinload
+from sqlalchemy.orm import selectinload
 
-import dstack._internal.server.services.jobs as jobs_services
 import dstack._internal.utils.random_names as random_names
-from dstack._internal.core.backends import (
-    BACKENDS_WITH_GATEWAY_SUPPORT,
-    BACKENDS_WITH_PRIVATE_GATEWAY_SUPPORT,
-)
 from dstack._internal.core.backends.base.compute import (
     Compute,
+    ComputeWithGatewaySupport,
     get_dstack_gateway_wheel,
     get_dstack_runner_version,
 )
+from dstack._internal.core.backends.features import (
+    BACKENDS_WITH_GATEWAY_SUPPORT,
+    BACKENDS_WITH_PRIVATE_GATEWAY_SUPPORT,
+)
 from dstack._internal.core.errors import (
     GatewayError,
     ResourceNotExistsError,
@@ -28,60 +28,47 @@ from dstack._internal.core.errors import (
     SSHError,
 )
 from dstack._internal.core.models.backends.base import BackendType
-from dstack._internal.core.models.common import is_core_model_instance
-from dstack._internal.core.models.configurations import SERVICE_HTTPS_DEFAULT, ServiceConfiguration
 from dstack._internal.core.models.gateways import (
     Gateway,
     GatewayComputeConfiguration,
     GatewayConfiguration,
+    GatewaySpec,
     GatewayStatus,
     LetsEncryptGatewayCertificate,
 )
-from dstack._internal.core.models.runs import (
-    Run,
-    RunSpec,
-    ServiceModelSpec,
-    ServiceSpec,
-)
 from dstack._internal.core.services import validate_dstack_resource_name
 from dstack._internal.server import settings
 from dstack._internal.server.db import get_db
 from dstack._internal.server.models import (
     GatewayComputeModel,
     GatewayModel,
-    JobModel,
     ProjectModel,
-    RunModel,
+    UserModel,
 )
 from dstack._internal.server.services.backends import (
+    check_backend_type_available,
     get_project_backend_by_type_or_error,
     get_project_backend_with_model_by_type_or_error,
 )
 from dstack._internal.server.services.gateways.connection import GatewayConnection
-from dstack._internal.server.services.gateways.options import get_service_options
 from dstack._internal.server.services.gateways.pool import gateway_connections_pool
 from dstack._internal.server.services.locking import (
     advisory_lock_ctx,
     get_locker,
     string_to_lock_id,
 )
-from dstack._internal.server.services.logging import fmt
-from dstack._internal.server.utils.common import (
-    gather_map_async,
-    run_async,
-)
-from dstack._internal.utils.common import get_current_datetime
+from dstack._internal.server.services.plugins import apply_plugin_policies
+from dstack._internal.server.utils.common import gather_map_async
+from dstack._internal.utils.common import get_current_datetime, run_async
 from dstack._internal.utils.crypto import generate_rsa_key_pair_bytes
 from dstack._internal.utils.logging import get_logger
 
-# TODO(#1595): refactor into different modules: gateway-specific and proxy-specific
-
 logger = get_logger(__name__)
 
 
 GATEWAY_CONNECT_ATTEMPTS = 30
 GATEWAY_CONNECT_DELAY = 10
-GATEWAY_CONFIGURE_ATTEMPTS = 40
+GATEWAY_CONFIGURE_ATTEMPTS = 50
 GATEWAY_CONFIGURE_DELAY = 3
 
 
@@ -99,21 +86,13 @@ async def get_gateway_by_name(
     return gateway_model_to_gateway(gateway)
 
 
-async def get_project_default_gateway(
-    session: AsyncSession, project: ProjectModel
-) -> Optional[Gateway]:
-    gateway: Optional[GatewayModel] = project.default_gateway
-    if gateway is None:
-        return None
-    return gateway_model_to_gateway(gateway)
-
-
 async def create_gateway_compute(
     project_name: str,
     backend_compute: Compute,
     configuration: GatewayConfiguration,
     backend_id: Optional[uuid.UUID] = None,
 ) -> GatewayComputeModel:
+    assert isinstance(backend_compute, ComputeWithGatewaySupport)
     private_bytes, public_bytes = generate_rsa_key_pair_bytes()
     gateway_ssh_private_key = private_bytes.decode()
     gateway_ssh_public_key = public_bytes.decode()
@@ -126,6 +105,7 @@ async def create_gateway_compute(
         public_ip=configuration.public_ip,
         ssh_key_pub=gateway_ssh_public_key,
         certificate=configuration.certificate,
+        tags=configuration.tags,
     )
 
     gpd = await run_async(
@@ -148,9 +128,17 @@ async def create_gateway_compute(
 
 async def create_gateway(
     session: AsyncSession,
+    user: UserModel,
     project: ProjectModel,
     configuration: GatewayConfiguration,
 ) -> Gateway:
+    spec = await apply_plugin_policies(
+        user=user.name,
+        project=project.name,
+        # Create pseudo spec until the gateway API is updated to accept spec
+        spec=GatewaySpec(configuration=configuration),
+    )
+    configuration = spec.configuration
     _validate_gateway_configuration(configuration)
 
     backend_model, _ = await get_project_backend_with_model_by_type_or_error(
@@ -159,14 +147,14 @@ async def create_gateway(
 
     lock_namespace = f"gateway_names_{project.name}"
     if get_db().dialect_name == "sqlite":
-        # Start new transaction to see commited changes after lock
+        # Start new transaction to see committed changes after lock
         await session.commit()
     elif get_db().dialect_name == "postgresql":
         await session.execute(
             select(func.pg_advisory_xact_lock(string_to_lock_id(lock_namespace)))
         )
 
-    lock, _ = get_locker().get_lockset(lock_namespace)
+    lock, _ = get_locker(get_db().dialect_name).get_lockset(lock_namespace)
     async with lock:
         if configuration.name is None:
             configuration.name = await generate_gateway_name(session=session, project=project)
@@ -184,12 +172,13 @@ async def create_gateway(
         session.add(gateway)
         await session.commit()
 
-        if project.default_gateway is None or configuration.default:
+        default_gateway = await get_project_default_gateway_model(session=session, project=project)
+        if default_gateway is None or configuration.default:
             await set_default_gateway(session=session, project=project, name=configuration.name)
-
         return gateway_model_to_gateway(gateway)
 
 
+# NOTE: dstack Sky imports and uses this function
 async def connect_to_gateway_with_retry(
     gateway_compute: GatewayComputeModel,
 ) -> Optional[GatewayConnection]:
@@ -233,7 +222,9 @@ async def delete_gateways(
     gateways_ids = sorted([g.id for g in gateway_models])
     await session.commit()
     logger.info("Deleting gateways: %s", [g.name for g in gateway_models])
-    async with get_locker().lock_ctx(GatewayModel.__tablename__, gateways_ids):
+    async with get_locker(get_db().dialect_name).lock_ctx(
+        GatewayModel.__tablename__, gateways_ids
+    ):
         # Refetch after lock
         res = await session.execute(
             select(GatewayModel)
@@ -243,13 +234,16 @@ async def delete_gateways(
             )
             .options(selectinload(GatewayModel.gateway_compute))
             .execution_options(populate_existing=True)
-            .with_for_update()
+            .order_by(GatewayModel.id)  # take locks in order
+            .with_for_update(key_share=True)
         )
         gateway_models = res.scalars().all()
         for gateway_model in gateway_models:
             backend = await get_project_backend_by_type_or_error(
                 project=project, backend_type=gateway_model.backend.type
             )
+            compute = backend.compute()
+            assert isinstance(compute, ComputeWithGatewaySupport)
             gateway_compute_configuration = get_gateway_compute_configuration(gateway_model)
             if (
                 gateway_model.gateway_compute is not None
@@ -258,7 +252,7 @@ async def delete_gateways(
                 logger.info("Deleting gateway compute for %s...", gateway_model.name)
                 try:
                     await run_async(
-                        backend.compute().terminate_gateway,
+                        compute.terminate_gateway,
                         gateway_model.gateway_compute.instance_id,
                         gateway_compute_configuration,
                         gateway_model.gateway_compute.backend_data,
@@ -346,6 +340,15 @@ async def get_project_gateway_model_by_name(
     return res.scalar()
 
 
+async def get_project_default_gateway_model(
+    session: AsyncSession, project: ProjectModel
+) -> Optional[GatewayModel]:
+    res = await session.execute(
+        select(GatewayModel).where(GatewayModel.id == project.default_gateway_id)
+    )
+    return res.scalar_one_or_none()
+
+
 async def generate_gateway_name(session: AsyncSession, project: ProjectModel) -> str:
     gateways = await list_project_gateway_models(session=session, project=project)
     names = {g.name for g in gateways}
@@ -355,201 +358,6 @@ async def generate_gateway_name(session: AsyncSession, project: ProjectModel) ->
             return name
 
 
-async def register_service(session: AsyncSession, run_model: RunModel, run_spec: RunSpec):
-    assert is_core_model_instance(run_spec.configuration, ServiceConfiguration)
-
-    if isinstance(run_spec.configuration.gateway, str):
-        gateway = await get_project_gateway_model_by_name(
-            session=session, project=run_model.project, name=run_spec.configuration.gateway
-        )
-        if gateway is None:
-            raise ResourceNotExistsError(
-                f"Gateway {run_spec.configuration.gateway} does not exist"
-            )
-    elif run_spec.configuration.gateway == False:
-        gateway = None
-    else:
-        gateway = run_model.project.default_gateway
-
-    if gateway is not None:
-        service_spec = await _register_service_in_gateway(session, run_model, run_spec, gateway)
-        run_model.gateway = gateway
-    elif not settings.FORBID_SERVICES_WITHOUT_GATEWAY:
-        service_spec = _register_service_in_server(run_model, run_spec)
-    else:
-        raise ResourceNotExistsError(
-            "This dstack-server installation forbids services without a gateway."
-            " Please configure a gateway."
-        )
-    run_model.service_spec = service_spec.json()
-
-
-async def _register_service_in_gateway(
-    session: AsyncSession, run_model: RunModel, run_spec: RunSpec, gateway: GatewayModel
-) -> ServiceSpec:
-    if gateway.gateway_compute is None:
-        raise ServerClientError("Gateway has no instance associated with it")
-
-    if gateway.status != GatewayStatus.RUNNING:
-        raise ServerClientError("Gateway status is not running")
-
-    gateway_configuration = get_gateway_configuration(gateway)
-    service_https = _get_service_https(run_spec, gateway_configuration)
-    service_protocol = "https" if service_https else "http"
-
-    if service_https and gateway_configuration.certificate is None:
-        raise ServerClientError(
-            "Cannot run HTTPS service on gateway with no SSL certificates configured"
-        )
-
-    gateway_https = _get_gateway_https(gateway_configuration)
-    gateway_protocol = "https" if gateway_https else "http"
-
-    wildcard_domain = gateway.wildcard_domain.lstrip("*.") if gateway.wildcard_domain else None
-    if wildcard_domain is None:
-        raise ServerClientError("Domain is required for gateway")
-    service_spec = get_service_spec(
-        configuration=run_spec.configuration,
-        service_url=f"{service_protocol}://{run_model.run_name}.{wildcard_domain}",
-        model_url=f"{gateway_protocol}://gateway.{wildcard_domain}",
-    )
-
-    conn = await get_or_add_gateway_connection(session, gateway.id)
-    try:
-        logger.debug("%s: registering service as %s", fmt(run_model), service_spec.url)
-        async with conn.client() as client:
-            await client.register_service(
-                project=run_model.project.name,
-                run_id=run_model.id,
-                domain=urlparse(service_spec.url).hostname,
-                service_https=service_https,
-                gateway_https=gateway_https,
-                auth=run_spec.configuration.auth,
-                client_max_body_size=settings.DEFAULT_SERVICE_CLIENT_MAX_BODY_SIZE,
-                options=service_spec.options,
-                ssh_private_key=run_model.project.ssh_private_key,
-            )
-        logger.info("%s: service is registered as %s", fmt(run_model), service_spec.url)
-    except SSHError:
-        raise ServerClientError("Gateway tunnel is not working")
-    except httpx.RequestError as e:
-        logger.debug("Gateway request failed", exc_info=True)
-        raise GatewayError(f"Gateway is not working: {e!r}")
-
-    return service_spec
-
-
-def _register_service_in_server(run_model: RunModel, run_spec: RunSpec) -> ServiceSpec:
-    if run_spec.configuration.https != SERVICE_HTTPS_DEFAULT:
-        # Note: if the user sets `https: <default-value>`, it will be ignored silently
-        # TODO: in 0.19, make `https` Optional to be able to tell if it was set or omitted
-        raise ServerClientError(
-            "The `https` configuration property is not applicable when running services without a gateway. "
-            "Please configure a gateway or remove the `https` property from the service configuration"
-        )
-    if run_spec.configuration.replicas.min != run_spec.configuration.replicas.max:
-        raise ServerClientError(
-            "Auto-scaling is not yet supported when running services without a gateway. "
-            "Please configure a gateway or set `replicas` to a fixed value in the service configuration"
-        )
-    return get_service_spec(
-        configuration=run_spec.configuration,
-        service_url=f"/proxy/services/{run_model.project.name}/{run_model.run_name}/",
-        model_url=f"/proxy/models/{run_model.project.name}/",
-    )
-
-
-def get_service_spec(
-    configuration: ServiceConfiguration, service_url: str, model_url: str
-) -> ServiceSpec:
-    service_spec = ServiceSpec(url=service_url)
-    if configuration.model is not None:
-        service_spec.model = ServiceModelSpec(
-            name=configuration.model.name,
-            base_url=model_url,
-            type=configuration.model.type,
-        )
-        service_spec.options = get_service_options(configuration)
-    return service_spec
-
-
-async def register_replica(
-    session: AsyncSession, gateway_id: Optional[uuid.UUID], run: Run, job_model: JobModel
-):
-    if gateway_id is None:  # in-server proxy
-        return
-    conn = await get_or_add_gateway_connection(session, gateway_id)
-    job_submission = jobs_services.job_model_to_job_submission(job_model)
-    try:
-        logger.debug("%s: registering replica for service %s", fmt(job_model), run.id.hex)
-        async with conn.client() as client:
-            await client.register_replica(
-                run=run,
-                job_submission=job_submission,
-            )
-        logger.info("%s: replica is registered for service %s", fmt(job_model), run.id.hex)
-    except (httpx.RequestError, SSHError) as e:
-        logger.debug("Gateway request failed", exc_info=True)
-        raise GatewayError(repr(e))
-
-
-async def unregister_service(session: AsyncSession, run_model: RunModel):
-    if run_model.gateway_id is None:  # in-server proxy
-        return
-    conn = await get_or_add_gateway_connection(session, run_model.gateway_id)
-    res = await session.execute(
-        select(ProjectModel).where(ProjectModel.id == run_model.project_id)
-    )
-    project = res.scalar_one()
-    try:
-        logger.debug("%s: unregistering service", fmt(run_model))
-        async with conn.client() as client:
-            await client.unregister_service(
-                project=project.name,
-                run_id=run_model.id,
-            )
-        logger.debug("%s: service is unregistered", fmt(run_model))
-    except GatewayError as e:
-        # ignore if service is not registered
-        logger.warning("%s: unregistering service: %s", fmt(run_model), e)
-    except (httpx.RequestError, SSHError) as e:
-        logger.debug("Gateway request failed", exc_info=True)
-        raise GatewayError(repr(e))
-
-
-async def unregister_replica(session: AsyncSession, job_model: JobModel):
-    res = await session.execute(
-        select(RunModel)
-        .where(RunModel.id == job_model.run_id)
-        .options(joinedload(RunModel.project).joinedload(ProjectModel.backends))
-    )
-    run_model = res.unique().scalar_one()
-    if run_model.gateway_id is None:
-        # not a service or served by in-server proxy
-        return
-
-    conn = await get_or_add_gateway_connection(session, run_model.gateway_id)
-    try:
-        logger.debug(
-            "%s: unregistering replica from service %s", fmt(job_model), job_model.run_id.hex
-        )
-        async with conn.client() as client:
-            await client.unregister_replica(
-                project=run_model.project.name,
-                run_id=run_model.id,
-                job_id=job_model.id,
-            )
-        logger.info(
-            "%s: replica is unregistered from service %s", fmt(job_model), job_model.run_id.hex
-        )
-    except GatewayError as e:
-        # ignore if replica is not registered
-        logger.warning("%s: unregistering replica from service: %s", fmt(job_model), e)
-    except (httpx.RequestError, SSHError) as e:
-        logger.debug("Gateway request failed", exc_info=True)
-        raise GatewayError(repr(e))
-
-
 async def get_or_add_gateway_connection(
     session: AsyncSession, gateway_id: uuid.UUID
 ) -> GatewayConnection:
@@ -574,6 +382,8 @@ async def get_or_add_gateway_connection(
 async def init_gateways(session: AsyncSession):
     res = await session.execute(
         select(GatewayComputeModel).where(
+            # FIXME: should not include computes related to gateways in the `provisioning` status.
+            # Causes warnings and delays when restarting the server during gateway provisioning.
             GatewayComputeModel.active == True,
             GatewayComputeModel.deleted == False,
         )
@@ -615,7 +425,8 @@ async def init_gateways(session: AsyncSession):
 
         for gateway_compute, error in await gather_map_async(
             await gateway_connections_pool.all(),
-            configure_gateway,
+            # Need several attempts to handle short gateway downtime after update
+            partial(configure_gateway, attempts=7),
             return_exceptions=True,
         ):
             if isinstance(error, Exception):
@@ -636,9 +447,13 @@ async def _update_gateway(gateway_compute_model: GatewayComputeModel, build: str
         gateway_compute_model.ssh_private_key,
     )
     logger.debug("Updating gateway %s", connection.ip_address)
-    stdout = await connection.tunnel.aexec(
-        f"/bin/sh dstack/update.sh {get_dstack_gateway_wheel(build)} {build}"
-    )
+    commands = [
+        # prevent update.sh from overwriting itself during execution
+        "cp dstack/update.sh dstack/_update.sh",
+        f"sh dstack/_update.sh {get_dstack_gateway_wheel(build)} {build}",
+        "rm dstack/_update.sh",
+    ]
+    stdout = await connection.tunnel.aexec("/bin/sh -c '" + " && ".join(commands) + "'")
     if "Update successfully completed" in stdout:
         logger.info("Gateway %s updated", connection.ip_address)
         return True
@@ -651,7 +466,11 @@ def _recently_updated(gateway_compute_model: GatewayComputeModel) -> bool:
     ) > get_current_datetime() - timedelta(seconds=60)
 
 
-async def configure_gateway(connection: GatewayConnection) -> None:
+# NOTE: dstack Sky imports and uses this function
+async def configure_gateway(
+    connection: GatewayConnection,
+    attempts: int = GATEWAY_CONFIGURE_ATTEMPTS,
+) -> None:
     """
     Try submitting gateway config several times in case gateway's HTTP server is not
     running yet
@@ -659,7 +478,7 @@ async def configure_gateway(connection: GatewayConnection) -> None:
 
     logger.debug("Configuring gateway %s", connection.ip_address)
 
-    for attempt in range(GATEWAY_CONFIGURE_ATTEMPTS - 1):
+    for attempt in range(attempts - 1):
         try:
             async with connection.client() as client:
                 await client.submit_gateway_config()
@@ -668,7 +487,7 @@ async def configure_gateway(connection: GatewayConnection) -> None:
             logger.debug(
                 "Failed attempt %s/%s at configuring gateway %s: %r",
                 attempt + 1,
-                GATEWAY_CONFIGURE_ATTEMPTS,
+                attempts,
                 connection.ip_address,
                 e,
             )
@@ -738,7 +557,7 @@ def gateway_model_to_gateway(gateway_model: GatewayModel) -> Gateway:
         region=gateway_model.region,
         wildcard_domain=gateway_model.wildcard_domain,
         default=gateway_model.project.default_gateway_id == gateway_model.id,
-        created_at=gateway_model.created_at.replace(tzinfo=timezone.utc),
+        created_at=gateway_model.created_at,
         status=gateway_model.status,
         status_message=gateway_model.status_message,
         configuration=configuration,
@@ -746,10 +565,12 @@ def gateway_model_to_gateway(gateway_model: GatewayModel) -> Gateway:
 
 
 def _validate_gateway_configuration(configuration: GatewayConfiguration):
+    check_backend_type_available(configuration.backend)
     if configuration.backend not in BACKENDS_WITH_GATEWAY_SUPPORT:
         raise ServerClientError(
-            f"Gateways are not supported for {configuration.backend.value} backend. "
-            f"Supported backends: {[b.value for b in BACKENDS_WITH_GATEWAY_SUPPORT]}."
+            f"Gateways are not supported for {configuration.backend.value} backend."
+            " Available backends with gateway support:"
+            f" {[b.value for b in BACKENDS_WITH_GATEWAY_SUPPORT]}."
         )
 
     if configuration.name is not None:
@@ -761,7 +582,8 @@ def _validate_gateway_configuration(configuration: GatewayConfiguration):
     ):
         raise ServerClientError(
             f"Private gateways are not supported for {configuration.backend.value} backend. "
-            f"Supported backends: {[b.value for b in BACKENDS_WITH_PRIVATE_GATEWAY_SUPPORT]}."
+            " Available backends with private gateway support:"
+            f" {[b.value for b in BACKENDS_WITH_PRIVATE_GATEWAY_SUPPORT]}."
         )
 
     if configuration.certificate is not None:
@@ -771,19 +593,3 @@ def _validate_gateway_configuration(configuration: GatewayConfiguration):
             )
         if configuration.certificate.type == "acm" and configuration.backend != BackendType.AWS:
             raise ServerClientError("acm certificate type is supported for aws backend only")
-
-
-def _get_service_https(run_spec: RunSpec, configuration: GatewayConfiguration) -> bool:
-    if not run_spec.configuration.https:
-        return False
-    if configuration.certificate is not None and configuration.certificate.type == "acm":
-        return False
-    return True
-
-
-def _get_gateway_https(configuration: GatewayConfiguration) -> bool:
-    if configuration.certificate is not None and configuration.certificate.type == "acm":
-        return False
-    if configuration.certificate is not None and configuration.certificate.type == "lets-encrypt":
-        return True
-    return False
