@@ -1,24 +1,18 @@
 import asyncio
 import uuid
-from typing import Dict, Optional
+from typing import Optional
 
 import httpx
-from pydantic import BaseModel, parse_obj_as
+from pydantic import parse_obj_as
 
+from dstack._internal.core.consts import DSTACK_RUNNER_SSH_PORT
 from dstack._internal.core.errors import GatewayError
-from dstack._internal.core.models.runs import JobSubmission, Run
+from dstack._internal.core.models.configurations import RateLimit, ServiceConfiguration
+from dstack._internal.core.models.instances import SSHConnectionParams
+from dstack._internal.core.models.runs import JobSpec, JobSubmission, Run, get_service_port
+from dstack._internal.proxy.gateway.schemas.stats import ServiceStats
 from dstack._internal.server import settings
 
-GATEWAY_MANAGEMENT_PORT = 8000
-
-
-class Stat(BaseModel):
-    requests: int
-    request_time: float
-
-
-StatsCollectResponse = Dict[str, Dict[int, Stat]]
-
 
 class GatewayClient:
     def __init__(self, uds: Optional[str] = None, port: Optional[int] = None):
@@ -42,13 +36,14 @@ class GatewayClient:
     async def register_service(
         self,
         project: str,
-        run_id: uuid.UUID,
+        run_name: str,
         domain: str,
         service_https: bool,
         gateway_https: bool,
         auth: bool,
         client_max_body_size: int,
         options: dict,
+        rate_limits: list[RateLimit],
         ssh_private_key: str,
     ):
         if "openai" in options:
@@ -56,12 +51,13 @@ class GatewayClient:
             await self.register_openai_entrypoint(project, entrypoint, gateway_https)
 
         payload = {
-            "run_id": run_id.hex,
+            "run_name": run_name,
             "domain": domain,
             "https": service_https,
             "auth": auth,
             "client_max_body_size": client_max_body_size,
             "options": options,
+            "rate_limits": [limit.dict() for limit in rate_limits],
             "ssh_private_key": ssh_private_key,
         }
         resp = await self._client.post(
@@ -72,19 +68,29 @@ class GatewayClient:
         resp.raise_for_status()
         self.is_server_ready = True
 
-    async def unregister_service(self, project: str, run_id: uuid.UUID):
+    async def unregister_service(self, project: str, run_name: str):
         resp = await self._client.post(
-            self._url(f"/api/registry/{project}/services/{run_id.hex}/unregister")
+            self._url(f"/api/registry/{project}/services/{run_name}/unregister")
         )
         if resp.status_code == 400:
             raise gateway_error(resp.json())
         resp.raise_for_status()
         self.is_server_ready = True
 
-    async def register_replica(self, run: Run, job_submission: JobSubmission):
+    async def register_replica(
+        self,
+        run: Run,
+        job_spec: JobSpec,
+        job_submission: JobSubmission,
+        ssh_head_proxy: Optional[SSHConnectionParams],
+        ssh_head_proxy_private_key: Optional[str],
+    ):
+        assert isinstance(run.run_spec.configuration, ServiceConfiguration)
         payload = {
             "job_id": job_submission.id.hex,
-            "app_port": run.run_spec.configuration.port.container_port,
+            "app_port": get_service_port(job_spec, run.run_spec.configuration),
+            "ssh_head_proxy": ssh_head_proxy.dict() if ssh_head_proxy is not None else None,
+            "ssh_head_proxy_private_key": ssh_head_proxy_private_key,
         }
         jpd = job_submission.job_provisioning_data
         if not jpd.dockerized:
@@ -92,27 +98,29 @@ class GatewayClient:
                 {
                     "ssh_port": jpd.ssh_port,
                     "ssh_host": f"{jpd.username}@{jpd.hostname}",
+                    "ssh_proxy": jpd.ssh_proxy.dict() if jpd.ssh_proxy is not None else None,
                 }
             )
-            if jpd.ssh_proxy is not None:
-                payload.update(
-                    {
-                        "ssh_jump_port": jpd.ssh_proxy.port,
-                        "ssh_jump_host": f"{jpd.ssh_proxy.username}@{jpd.ssh_proxy.hostname}",
-                    }
-                )
         else:
+            ssh_port = DSTACK_RUNNER_SSH_PORT
+            jrd = job_submission.job_runtime_data
+            if jrd is not None and jrd.ports is not None:
+                ssh_port = jrd.ports.get(ssh_port, ssh_port)
             payload.update(
                 {
-                    "ssh_port": 10022,
+                    "ssh_port": ssh_port,
                     "ssh_host": "root@localhost",
-                    "ssh_jump_port": jpd.ssh_port,
-                    "ssh_jump_host": f"{jpd.username}@{jpd.hostname}",
+                    "ssh_proxy": SSHConnectionParams(
+                        hostname=jpd.hostname,
+                        username=jpd.username,
+                        port=jpd.ssh_port,
+                    ).dict(),
                 }
             )
-
         resp = await self._client.post(
-            self._url(f"/api/registry/{run.project_name}/services/{run.id.hex}/replicas/register"),
+            self._url(
+                f"/api/registry/{run.project_name}/services/{run.run_spec.run_name}/replicas/register"
+            ),
             json=payload,
         )
         if resp.status_code == 400:
@@ -120,10 +128,10 @@ class GatewayClient:
         resp.raise_for_status()
         self.is_server_ready = True
 
-    async def unregister_replica(self, project: str, run_id: uuid.UUID, job_id: uuid.UUID):
+    async def unregister_replica(self, project: str, run_name: str, job_id: uuid.UUID):
         resp = await self._client.post(
             self._url(
-                f"/api/registry/{project}/services/{run_id.hex}/replicas/{job_id.hex}/unregister"
+                f"/api/registry/{project}/services/{run_name}/replicas/{job_id.hex}/unregister"
             )
         )
         if resp.status_code == 400:
@@ -135,7 +143,6 @@ class GatewayClient:
         resp = await self._client.post(
             self._url(f"/api/registry/{project}/entrypoints/register"),
             json={
-                "module": "openai",
                 "domain": domain,
                 "https": https,
             },
@@ -167,20 +174,25 @@ class GatewayClient:
         self.is_server_ready = True
         return resp.json()
 
-    async def collect_stats(self) -> StatsCollectResponse:
+    async def collect_stats(self) -> list[ServiceStats]:
         resp = await self._client.get(self._url("/api/stats/collect"))
         if resp.status_code == 400:
             raise gateway_error(resp.json())
         resp.raise_for_status()
         self.is_server_ready = True
-        return parse_obj_as(StatsCollectResponse, resp.json())
+        resp_data = resp.json()
+        if isinstance(resp_data, dict):
+            # Avoid errors if gateway is updated to new format and current server replica isn't.
+            # TODO: remove after a few releases
+            return []
+        return parse_obj_as(list[ServiceStats], resp_data)
 
     def _url(self, path: str) -> str:
         return f"{self.base_url}/{path.lstrip('/')}"
 
 
 def gateway_error(data: dict) -> GatewayError:
-    return GatewayError(msg=f"{data['error']}: {data['message']}")
+    return GatewayError(msg=data["detail"])
 
 
 class AsyncClientWrapper(httpx.AsyncClient):
