@@ -5,8 +5,6 @@ from dstack._internal.core.models.profiles import SpotPolicy
 from dstack._internal.core.models.runs import JobSpec
 from dstack._internal.server.services.jobs.configurators.base import JobConfigurator
 
-DEFAULT_MAX_DURATION_SECONDS = 72 * 3600
-
 
 class TaskJobConfigurator(JobConfigurator):
     TYPE: RunConfigurationType = RunConfigurationType.TASK
@@ -25,8 +23,11 @@ class TaskJobConfigurator(JobConfigurator):
     def _shell_commands(self) -> List[str]:
         return self.run_spec.configuration.commands
 
+    def _default_single_branch(self) -> bool:
+        return True
+
     def _default_max_duration(self) -> Optional[int]:
-        return DEFAULT_MAX_DURATION_SECONDS
+        return None
 
     def _spot_policy(self) -> SpotPolicy:
         return self.run_spec.merged_profile.spot_policy or SpotPolicy.ONDEMAND
