@@ -1,14 +1,14 @@
 import hashlib
 import os
+import re
 import uuid
-from datetime import timezone
 from typing import Awaitable, Callable, List, Optional, Tuple
 
 from sqlalchemy import delete, select, update
 from sqlalchemy import func as safunc
 from sqlalchemy.ext.asyncio import AsyncSession
 
-from dstack._internal.core.errors import ResourceExistsError
+from dstack._internal.core.errors import ResourceExistsError, ServerClientError
 from dstack._internal.core.models.users import (
     GlobalRole,
     User,
@@ -78,6 +78,7 @@ async def create_user(
     active: bool = True,
     token: Optional[str] = None,
 ) -> UserModel:
+    validate_username(username)
     user_model = await get_user_model_by_name(session=session, username=username, ignore_case=True)
     if user_model is not None:
         raise ResourceExistsError()
@@ -146,6 +147,7 @@ async def delete_users(
 ):
     await session.execute(delete(UserModel).where(UserModel.name.in_(usernames)))
     await session.commit()
+    logger.info("Deleted users %s by user %s", usernames, user.name)
 
 
 async def get_user_model_by_name(
@@ -192,7 +194,7 @@ def user_model_to_user(user_model: UserModel) -> User:
     return User(
         id=user_model.id,
         username=user_model.name,
-        created_at=user_model.created_at.replace(tzinfo=timezone.utc),
+        created_at=user_model.created_at,
         global_role=user_model.global_role,
         email=user_model.email,
         active=user_model.active,
@@ -204,7 +206,7 @@ def user_model_to_user_with_creds(user_model: UserModel) -> UserWithCreds:
     return UserWithCreds(
         id=user_model.id,
         username=user_model.name,
-        created_at=user_model.created_at.replace(tzinfo=timezone.utc),
+        created_at=user_model.created_at,
         global_role=user_model.global_role,
         email=user_model.email,
         active=user_model.active,
@@ -224,6 +226,15 @@ def get_user_permissions(user_model: UserModel) -> UserPermissions:
     )
 
 
+def validate_username(username: str):
+    if not is_valid_username(username):
+        raise ServerClientError("Username should match regex '^[a-zA-Z0-9-_]{1,60}$'")
+
+
+def is_valid_username(username: str) -> bool:
+    return re.match("^[a-zA-Z0-9-_]{1,60}$", username) is not None
+
+
 _CREATE_USER_HOOKS = []
 
 
