@@ -1,25 +1,52 @@
 import json
 import uuid
+from collections.abc import Callable
 from contextlib import contextmanager
 from datetime import datetime, timezone
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Literal, Optional, Union
 from uuid import UUID
 
+import gpuhunt
 from sqlalchemy.ext.asyncio import AsyncSession
 
+from dstack._internal.core.backends.base.compute import (
+    Compute,
+    ComputeWithCreateInstanceSupport,
+    ComputeWithGatewaySupport,
+    ComputeWithMultinodeSupport,
+    ComputeWithPlacementGroupSupport,
+    ComputeWithPrivateGatewaySupport,
+    ComputeWithReservationSupport,
+    ComputeWithVolumeSupport,
+)
 from dstack._internal.core.models.backends.base import BackendType
+from dstack._internal.core.models.common import NetworkMode
 from dstack._internal.core.models.configurations import (
     AnyRunConfiguration,
     DevEnvironmentConfiguration,
 )
-from dstack._internal.core.models.fleets import FleetConfiguration, FleetSpec, FleetStatus
-from dstack._internal.core.models.gateways import GatewayStatus
+from dstack._internal.core.models.envs import Env
+from dstack._internal.core.models.fleets import (
+    FleetConfiguration,
+    FleetSpec,
+    FleetStatus,
+    InstanceGroupPlacement,
+    SSHHostParams,
+    SSHParams,
+)
+from dstack._internal.core.models.gateways import GatewayComputeConfiguration, GatewayStatus
+from dstack._internal.core.models.health import HealthStatus
 from dstack._internal.core.models.instances import (
+    Disk,
+    Gpu,
+    InstanceAvailability,
     InstanceConfiguration,
+    InstanceOfferWithAvailability,
     InstanceStatus,
     InstanceType,
     RemoteConnectionInfo,
     Resources,
+    SSHKey,
 )
 from dstack._internal.core.models.placement import (
     PlacementGroupConfiguration,
@@ -27,26 +54,27 @@ from dstack._internal.core.models.placement import (
     PlacementStrategy,
 )
 from dstack._internal.core.models.profiles import (
-    DEFAULT_POOL_NAME,
-    DEFAULT_POOL_TERMINATION_IDLE_TIME,
+    DEFAULT_FLEET_TERMINATION_IDLE_TIME,
     Profile,
     TerminationPolicy,
 )
 from dstack._internal.core.models.repos.base import RepoType
 from dstack._internal.core.models.repos.local import LocalRunRepoData
-from dstack._internal.core.models.resources import Memory, Range, ResourcesSpec
+from dstack._internal.core.models.resources import CPUSpec, Memory, Range, ResourcesSpec
 from dstack._internal.core.models.runs import (
     JobProvisioningData,
+    JobRuntimeData,
     JobStatus,
     JobTerminationReason,
     Requirements,
     RunSpec,
     RunStatus,
+    RunTerminationReason,
 )
 from dstack._internal.core.models.users import GlobalRole
 from dstack._internal.core.models.volumes import (
     Volume,
-    VolumeAttachmentData,
+    VolumeAttachment,
     VolumeConfiguration,
     VolumeProvisioningData,
     VolumeStatus,
@@ -54,19 +82,24 @@ from dstack._internal.core.models.volumes import (
 from dstack._internal.server.models import (
     BackendModel,
     DecryptedString,
+    FileArchiveModel,
     FleetModel,
     GatewayComputeModel,
     GatewayModel,
+    InstanceHealthCheckModel,
     InstanceModel,
     JobMetricsPoint,
     JobModel,
+    JobPrometheusMetrics,
     PlacementGroupModel,
-    PoolModel,
+    ProbeModel,
     ProjectModel,
     RepoCredsModel,
     RepoModel,
     RunModel,
+    SecretModel,
     UserModel,
+    VolumeAttachmentModel,
     VolumeModel,
 )
 from dstack._internal.server.services.jobs import get_job_specs_from_run_spec
@@ -116,6 +149,7 @@ async def create_project(
     created_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
     ssh_private_key: str = "",
     ssh_public_key: str = "",
+    is_public: bool = False,
 ) -> ProjectModel:
     if owner is None:
         owner = await create_user(session=session, name="test_owner")
@@ -125,6 +159,7 @@ async def create_project(
         created_at=created_at,
         ssh_private_key=ssh_private_key,
         ssh_public_key=ssh_public_key,
+        is_public=is_public,
     )
     session.add(project)
     await session.commit()
@@ -170,9 +205,6 @@ async def create_repo(
     if info is None:
         info = {
             "repo_type": "remote",
-            "repo_host_name": "",
-            "repo_port": None,
-            "repo_user_name": "",
             "repo_name": "dstack",
         }
     repo = RepoModel(
@@ -195,7 +227,6 @@ async def create_repo_creds(
 ) -> RepoCredsModel:
     if creds is None:
         creds = {
-            "protocol": "https",
             "clone_url": "https://github.com/dstackai/dstack.git",
             "private_key": None,
             "oauth_token": "test_token",
@@ -210,21 +241,38 @@ async def create_repo_creds(
     return repo_creds
 
 
+async def create_file_archive(
+    session: AsyncSession,
+    user_id: UUID,
+    blob_hash: str = "blob_hash",
+    blob: bytes = b"blob_content",
+) -> FileArchiveModel:
+    archive = FileArchiveModel(
+        user_id=user_id,
+        blob_hash=blob_hash,
+        blob=blob,
+    )
+    session.add(archive)
+    await session.commit()
+    return archive
+
+
 def get_run_spec(
     run_name: str,
     repo_id: str,
-    profile: Optional[Profile] = None,
+    configuration_path: str = "dstack.yaml",
+    profile: Union[Profile, Callable[[], Profile], None] = lambda: Profile(name="default"),
     configuration: Optional[AnyRunConfiguration] = None,
 ) -> RunSpec:
-    if profile is None:
-        profile = Profile(name="default")
+    if callable(profile):
+        profile = profile()
     return RunSpec(
         run_name=run_name,
         repo_id=repo_id,
         repo_data=LocalRunRepoData(repo_dir="/"),
         repo_code_hash=None,
         working_dir=".",
-        configuration_path="dstack.yaml",
+        configuration_path=configuration_path,
         configuration=configuration or DevEnvironmentConfiguration(ide="vscode"),
         profile=profile,
         ssh_key_pub="user_ssh_key",
@@ -238,10 +286,15 @@ async def create_run(
     user: UserModel,
     run_name: str = "test-run",
     status: RunStatus = RunStatus.SUBMITTED,
+    termination_reason: Optional[RunTerminationReason] = None,
     submitted_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
     run_spec: Optional[RunSpec] = None,
     run_id: Optional[UUID] = None,
     deleted: bool = False,
+    priority: int = 0,
+    deployment_num: int = 0,
+    resubmission_attempt: int = 0,
+    next_triggered_at: Optional[datetime] = None,
 ) -> RunModel:
     if run_spec is None:
         run_spec = get_run_spec(
@@ -259,9 +312,15 @@ async def create_run(
         submitted_at=submitted_at,
         run_name=run_name,
         status=status,
+        termination_reason=termination_reason,
         run_spec=run_spec.json(),
         last_processed_at=submitted_at,
         jobs=[],
+        priority=priority,
+        deployment_num=deployment_num,
+        desired_replica_count=1,
+        resubmission_attempt=resubmission_attempt,
+        next_triggered_at=next_triggered_at,
     )
     session.add(run)
     await session.commit()
@@ -277,20 +336,29 @@ async def create_job(
     last_processed_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
     termination_reason: Optional[JobTerminationReason] = None,
     job_provisioning_data: Optional[JobProvisioningData] = None,
+    job_runtime_data: Optional[JobRuntimeData] = None,
     instance: Optional[InstanceModel] = None,
     job_num: int = 0,
     replica_num: int = 0,
+    deployment_num: Optional[int] = None,
     instance_assigned: bool = False,
+    disconnected_at: Optional[datetime] = None,
 ) -> JobModel:
+    if deployment_num is None:
+        deployment_num = run.deployment_num
     run_spec = RunSpec.parse_raw(run.run_spec)
-    job_spec = (await get_job_specs_from_run_spec(run_spec, replica_num=replica_num))[0]
+    job_spec = (
+        await get_job_specs_from_run_spec(run_spec=run_spec, secrets={}, replica_num=replica_num)
+    )[0]
+    job_spec.job_num = job_num
     job = JobModel(
         project_id=run.project_id,
         run_id=run.id,
         run_name=run.run_name,
         job_num=job_num,
-        job_name=run.run_name + f"-0-{replica_num}",
+        job_name=run.run_name + f"-{job_num}-{replica_num}",
         replica_num=replica_num,
+        deployment_num=deployment_num,
         submission_num=submission_num,
         submitted_at=submitted_at,
         last_processed_at=last_processed_at,
@@ -298,35 +366,103 @@ async def create_job(
         termination_reason=termination_reason,
         job_spec_data=job_spec.json(),
         job_provisioning_data=job_provisioning_data.json() if job_provisioning_data else None,
+        job_runtime_data=job_runtime_data.json() if job_runtime_data else None,
         instance=instance,
         instance_assigned=instance_assigned,
         used_instance_id=instance.id if instance is not None else None,
+        disconnected_at=disconnected_at,
+        probes=[],
     )
     session.add(job)
     await session.commit()
     return job
 
 
-def get_job_provisioning_data() -> JobProvisioningData:
-    return JobProvisioningData(
-        backend=BackendType.AWS,
-        instance_type=InstanceType(
+def get_job_provisioning_data(
+    dockerized: bool = False,
+    backend: BackendType = BackendType.AWS,
+    region: str = "us-east-1",
+    gpu_count: int = 0,
+    gpu_memory_gib: float = 16,
+    gpu_name: str = "T4",
+    cpu_count: int = 1,
+    memory_gib: float = 0.5,
+    spot: bool = False,
+    hostname: str = "127.0.0.4",
+    internal_ip: Optional[str] = "127.0.0.4",
+    price: float = 10.5,
+    instance_type: Optional[InstanceType] = None,
+) -> JobProvisioningData:
+    gpus = [
+        Gpu(
+            name=gpu_name,
+            memory_mib=int(gpu_memory_gib * 1024),
+            vendor=gpuhunt.AcceleratorVendor.NVIDIA,
+        )
+    ] * gpu_count
+    if instance_type is None:
+        instance_type = InstanceType(
             name="instance",
-            resources=Resources(cpus=1, memory_mib=512, spot=False, gpus=[]),
-        ),
+            resources=Resources(
+                cpus=cpu_count, memory_mib=int(memory_gib * 1024), spot=spot, gpus=gpus
+            ),
+        )
+    return JobProvisioningData(
+        backend=backend,
+        instance_type=instance_type,
         instance_id="instance_id",
-        hostname="127.0.0.4",
-        internal_ip="127.0.0.4",
-        region="us-east-1",
-        price=10.5,
+        hostname=hostname,
+        internal_ip=internal_ip,
+        region=region,
+        price=price,
         username="ubuntu",
         ssh_port=22,
-        dockerized=False,
+        dockerized=dockerized,
         backend_data=None,
         ssh_proxy=None,
     )
 
 
+def get_job_runtime_data(
+    network_mode: str = NetworkMode.HOST,
+    cpu: Optional[float] = None,
+    gpu: Optional[int] = None,
+    memory: Optional[float] = None,
+    ports: Optional[dict[int, int]] = None,
+    offer: Optional[InstanceOfferWithAvailability] = None,
+    volume_names: Optional[list[str]] = None,
+) -> JobRuntimeData:
+    return JobRuntimeData(
+        network_mode=NetworkMode(network_mode),
+        cpu=cpu,
+        gpu=gpu,
+        memory=Memory(memory) if memory is not None else None,
+        ports=ports,
+        offer=offer,
+        volume_names=volume_names,
+    )
+
+
+async def create_probe(
+    session: AsyncSession,
+    job: JobModel,
+    probe_num: int = 0,
+    due: datetime = datetime(2025, 1, 2, 3, 4, tzinfo=timezone.utc),
+    success_streak: int = 0,
+) -> ProbeModel:
+    probe = ProbeModel(
+        name=f"{job.job_name}-{probe_num}",
+        job=job,
+        probe_num=probe_num,
+        due=due,
+        success_streak=success_streak,
+        active=True,
+    )
+    session.add(probe)
+    await session.commit()
+    return probe
+
+
 async def create_gateway(
     session: AsyncSession,
     project_id: UUID,
@@ -375,20 +511,22 @@ async def create_gateway_compute(
     return gateway_compute
 
 
-async def create_pool(
-    session: AsyncSession,
-    project: ProjectModel,
-    pool_name: Optional[str] = None,
-) -> PoolModel:
-    pool_name = pool_name if pool_name is not None else DEFAULT_POOL_NAME
-    pool = PoolModel(
-        name=pool_name,
-        project=project,
-        project_id=project.id,
+def get_gateway_compute_configuration(
+    project_name: str = "test-project",
+    instance_name: str = "test-instance",
+    backend: BackendType = BackendType.AWS,
+    region: str = "us",
+    public_ip: bool = True,
+) -> GatewayComputeConfiguration:
+    return GatewayComputeConfiguration(
+        project_name=project_name,
+        instance_name=instance_name,
+        backend=backend,
+        region=region,
+        public_ip=public_ip,
+        ssh_key_pub="",
+        certificate=None,
     )
-    session.add(pool)
-    await session.commit()
-    return pool
 
 
 async def create_fleet(
@@ -398,20 +536,27 @@ async def create_fleet(
     spec: Optional[FleetSpec] = None,
     fleet_id: Optional[UUID] = None,
     status: FleetStatus = FleetStatus.ACTIVE,
+    deleted: bool = False,
+    name: Optional[str] = None,
+    last_processed_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
 ) -> FleetModel:
     if fleet_id is None:
         fleet_id = uuid.uuid4()
     if spec is None:
         spec = get_fleet_spec()
+    if name is not None:
+        spec.configuration.name = name
     fm = FleetModel(
         id=fleet_id,
         project=project,
+        deleted=deleted,
         name=spec.configuration.name,
         status=status,
         created_at=created_at,
         spec=spec.json(),
         instances=[],
         runs=[],
+        last_processed_at=last_processed_at,
     )
     session.add(fm)
     await session.commit()
@@ -431,20 +576,47 @@ def get_fleet_spec(conf: Optional[FleetConfiguration] = None) -> FleetSpec:
 def get_fleet_configuration(
     name: str = "test-fleet",
     nodes: Range[int] = Range(min=1, max=1),
+    placement: Optional[InstanceGroupPlacement] = None,
 ) -> FleetConfiguration:
     return FleetConfiguration(
         name=name,
         nodes=nodes,
+        placement=placement,
+    )
+
+
+def get_ssh_fleet_configuration(
+    name: str = "test-fleet",
+    user: str = "ubuntu",
+    ssh_key: Optional[SSHKey] = None,
+    hosts: Optional[list[Union[SSHHostParams, str]]] = None,
+    network: Optional[str] = None,
+    placement: Optional[InstanceGroupPlacement] = None,
+) -> FleetConfiguration:
+    if ssh_key is None:
+        ssh_key = SSHKey(public="", private=get_private_key_string())
+    if hosts is None:
+        hosts = ["10.0.0.100"]
+    ssh_config = SSHParams(
+        user=user,
+        ssh_key=ssh_key,
+        hosts=hosts,
+        network=network,
+    )
+    return FleetConfiguration(
+        name=name,
+        ssh_config=ssh_config,
+        placement=placement,
     )
 
 
 async def create_instance(
     session: AsyncSession,
     project: ProjectModel,
-    pool: PoolModel,
     fleet: Optional[FleetModel] = None,
     status: InstanceStatus = InstanceStatus.IDLE,
     unreachable: bool = False,
+    health_status: HealthStatus = HealthStatus.HEALTHY,
     created_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
     finished_at: Optional[datetime] = None,
     spot: bool = False,
@@ -456,84 +628,64 @@ async def create_instance(
     instance_num: int = 0,
     backend: BackendType = BackendType.DATACRUNCH,
     termination_policy: Optional[TerminationPolicy] = None,
-    termination_idle_time: int = DEFAULT_POOL_TERMINATION_IDLE_TIME,
+    termination_idle_time: int = DEFAULT_FLEET_TERMINATION_IDLE_TIME,
     region: str = "eu-west",
     remote_connection_info: Optional[RemoteConnectionInfo] = None,
+    offer: Optional[Union[InstanceOfferWithAvailability, Literal["auto"]]] = "auto",
+    job_provisioning_data: Optional[Union[JobProvisioningData, Literal["auto"]]] = "auto",
+    total_blocks: Optional[int] = 1,
+    busy_blocks: int = 0,
+    name: str = "test_instance",
+    volumes: Optional[List[VolumeModel]] = None,
+    price: float = 1.0,
+    last_processed_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
 ) -> InstanceModel:
     if instance_id is None:
         instance_id = uuid.uuid4()
-    job_provisioning_data = {
-        "backend": backend.value,
-        "instance_type": {
-            "name": "instance",
-            "resources": {
-                "cpus": 1,
-                "memory_mib": 512,
-                "gpus": [],
-                "spot": spot,
-                "disk": {"size_mib": 102400},
-                "description": "",
-            },
-        },
-        "instance_id": "running_instance.id",
-        "ssh_proxy": None,
-        "hostname": "running_instance.ip",
-        "region": region,
-        "price": 0.1,
-        "username": "root",
-        "ssh_port": 22,
-        "dockerized": True,
-        "backend_data": None,
-    }
-    offer = {
-        "backend": backend.value,
-        "instance": {
-            "name": "instance",
-            "resources": {
-                "cpus": 2,
-                "memory_mib": 12000,
-                "gpus": [],
-                "spot": spot,
-                "disk": {"size_mib": 102400},
-                "description": "",
-            },
-        },
-        "region": region,
-        "price": 1,
-        "availability": "available",
-    }
-
+    if job_provisioning_data == "auto":
+        job_provisioning_data = get_job_provisioning_data(
+            dockerized=True,
+            backend=backend,
+            region=region,
+            spot=spot,
+            hostname="running_instance.ip",
+            internal_ip=None,
+        )
+    if offer == "auto":
+        offer = get_instance_offer_with_availability(
+            backend=backend, region=region, spot=spot, price=price
+        )
     if profile is None:
         profile = Profile(name="test_name")
 
     if requirements is None:
-        requirements = Requirements(resources=ResourcesSpec(cpu=1))
+        requirements = Requirements(resources=ResourcesSpec(cpu=CPUSpec.parse("1")))
 
     if instance_configuration is None:
-        instance_configuration = InstanceConfiguration(
-            project_name="test_proj",
-            instance_name="test_instance_name",
-            instance_id="test instance id",
-            job_docker_config=None,
-            ssh_keys=[],
-            user="test_user",
-        )
+        instance_configuration = get_instance_configuration()
+
+    if volumes is None:
+        volumes = []
+    volume_attachments = []
+    for volume in volumes:
+        volume_attachments.append(VolumeAttachmentModel(volume=volume))
 
     im = InstanceModel(
         id=instance_id,
-        name="test_instance",
+        name=name,
         instance_num=instance_num,
-        pool=pool,
         fleet=fleet,
         project=project,
         status=status,
+        last_processed_at=last_processed_at,
         unreachable=unreachable,
+        health=health_status,
         created_at=created_at,
         started_at=created_at,
         finished_at=finished_at,
-        job_provisioning_data=json.dumps(job_provisioning_data),
-        offer=json.dumps(offer),
-        price=1,
+        job_provisioning_data=job_provisioning_data.json() if job_provisioning_data else None,
+        offer=offer.json() if offer else None,
+        price=price,
         region=region,
         backend=backend,
         termination_policy=termination_policy,
@@ -542,19 +694,138 @@ async def create_instance(
         requirements=requirements.json(),
         instance_configuration=instance_configuration.json(),
         remote_connection_info=remote_connection_info.json() if remote_connection_info else None,
-        job=job,
+        volume_attachments=volume_attachments,
+        total_blocks=total_blocks,
+        busy_blocks=busy_blocks,
     )
+    if job:
+        im.jobs.append(job)
     session.add(im)
     await session.commit()
     return im
 
 
+def get_instance_configuration(
+    project_name: str = "test-project",
+    instance_name: str = "test-instance",
+    user: str = "dstack-user",
+) -> InstanceConfiguration:
+    return InstanceConfiguration(
+        project_name=project_name,
+        instance_name=instance_name,
+        user=user,
+        ssh_keys=[],
+    )
+
+
+def get_instance_offer_with_availability(
+    backend: BackendType = BackendType.AWS,
+    region: str = "eu-west",
+    gpu_count: int = 0,
+    gpu_name: str = "T4",
+    gpu_memory_gib: float = 16,
+    cpu_count: int = 2,
+    memory_gib: float = 12,
+    disk_gib: float = 100.0,
+    spot: bool = False,
+    blocks: int = 1,
+    total_blocks: int = 1,
+    availability_zones: Optional[List[str]] = None,
+    price: float = 1.0,
+    instance_type: str = "instance",
+):
+    gpus = [
+        Gpu(
+            name=gpu_name,
+            memory_mib=int(gpu_memory_gib * 1024),
+            vendor=gpuhunt.AcceleratorVendor.NVIDIA,
+        )
+    ] * gpu_count
+    return InstanceOfferWithAvailability(
+        backend=backend,
+        instance=InstanceType(
+            name=instance_type,
+            resources=Resources(
+                cpus=cpu_count,
+                memory_mib=int(memory_gib * 1024),
+                gpus=gpus,
+                spot=spot,
+                disk=Disk(size_mib=int(disk_gib * 1024)),
+                description="",
+            ),
+        ),
+        region=region,
+        price=price,
+        availability=InstanceAvailability.AVAILABLE,
+        availability_zones=availability_zones,
+        blocks=blocks,
+        total_blocks=total_blocks,
+    )
+
+
+def get_remote_connection_info(
+    host: str = "10.0.0.10",
+    port: int = 22,
+    ssh_user: str = "ubuntu",
+    ssh_keys: Optional[list[SSHKey]] = None,
+    env: Optional[Union[Env, dict]] = None,
+):
+    if ssh_keys is None:
+        ssh_keys = [get_ssh_key()]
+    if env is None:
+        env = Env()
+    elif isinstance(env, dict):
+        env = Env.parse_obj(env)
+    return RemoteConnectionInfo(
+        host=host,
+        port=port,
+        ssh_user=ssh_user,
+        ssh_keys=ssh_keys,
+        env=env,
+    )
+
+
+def get_ssh_key() -> SSHKey:
+    return SSHKey(
+        public="ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIO6mJxVbNtm0zXgMLvByrhXJCmJRveSrJxLB5/OzcyCk",
+        private="""
+                    -----BEGIN OPENSSH PRIVATE KEY-----
+                    b3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAAAMwAAAAtzc2gtZW
+                    QyNTUxOQAAACDupicVWzbZtM14DC7wcq4VyQpiUb3kqycSwefzs3MgpAAAAJCiWa5Volmu
+                    VQAAAAtzc2gtZWQyNTUxOQAAACDupicVWzbZtM14DC7wcq4VyQpiUb3kqycSwefzs3MgpA
+                    AAAEAncHi4AhS6XdMp5Gzd+IMse/4ekyQ54UngByf0Sp0uH+6mJxVbNtm0zXgMLvByrhXJ
+                    CmJRveSrJxLB5/OzcyCkAAAACWRlZkBkZWZwYwECAwQ=
+                    -----END OPENSSH PRIVATE KEY-----
+                """,
+    )
+
+
+async def create_instance_health_check(
+    session: AsyncSession,
+    instance: InstanceModel,
+    collected_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
+    status: HealthStatus = HealthStatus.HEALTHY,
+    response: str = "{}",
+) -> InstanceHealthCheckModel:
+    health_check = InstanceHealthCheckModel(
+        instance_id=instance.id,
+        collected_at=collected_at,
+        status=status,
+        response=response,
+    )
+    session.add(health_check)
+    await session.commit()
+    return health_check
+
+
 async def create_volume(
     session: AsyncSession,
     project: ProjectModel,
     user: UserModel,
     status: VolumeStatus = VolumeStatus.SUBMITTED,
     created_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
+    last_processed_at: Optional[datetime] = None,
+    last_job_processed_at: Optional[datetime] = None,
     configuration: Optional[VolumeConfiguration] = None,
     volume_provisioning_data: Optional[VolumeProvisioningData] = None,
     deleted_at: Optional[datetime] = None,
@@ -563,18 +834,23 @@ async def create_volume(
 ) -> VolumeModel:
     if configuration is None:
         configuration = get_volume_configuration(backend=backend, region=region)
+    if last_processed_at is None:
+        last_processed_at = created_at
     vm = VolumeModel(
         project=project,
         user_id=user.id,
         name=configuration.name,
         status=status,
         created_at=created_at,
+        last_processed_at=last_processed_at,
+        last_job_processed_at=last_job_processed_at,
         configuration=configuration.json(),
         volume_provisioning_data=volume_provisioning_data.json()
         if volume_provisioning_data
         else None,
-        instances=[],
+        attachments=[],
         deleted_at=deleted_at,
+        deleted=True if deleted_at else False,
     )
     session.add(vm)
     await session.commit()
@@ -589,17 +865,21 @@ def get_volume(
     configuration: Optional[VolumeConfiguration] = None,
     external: bool = False,
     created_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
+    last_processed_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
     status: VolumeStatus = VolumeStatus.ACTIVE,
     status_message: Optional[str] = None,
     deleted: bool = False,
+    deleted_at: Optional[datetime] = None,
     volume_id: Optional[str] = None,
     provisioning_data: Optional[VolumeProvisioningData] = None,
-    attachment_data: Optional[VolumeAttachmentData] = None,
+    attachments: Optional[List[VolumeAttachment]] = None,
 ) -> Volume:
     if id_ is None:
         id_ = uuid.uuid4()
     if configuration is None:
         configuration = get_volume_configuration()
+    if attachments is None:
+        attachments = []
     return Volume(
         id=id_,
         name=name,
@@ -608,12 +888,14 @@ def get_volume(
         configuration=configuration,
         external=external,
         created_at=created_at,
+        last_processed_at=last_processed_at,
         status=status,
         status_message=status_message,
         deleted=deleted,
+        deleted_at=deleted_at,
         volume_id=volume_id,
         provisioning_data=provisioning_data,
-        attachment_data=attachment_data,
+        attachments=attachments,
     )
 
 
@@ -623,6 +905,7 @@ def get_volume_configuration(
     region: str = "eu-west-1",
     size: Optional[Memory] = Memory(100),
     volume_id: Optional[str] = None,
+    auto_cleanup_duration: Optional[Union[str, int]] = None,
 ) -> VolumeConfiguration:
     return VolumeConfiguration(
         name=name,
@@ -630,6 +913,7 @@ def get_volume_configuration(
         region=region,
         size=size,
         volume_id=volume_id,
+        auto_cleanup_duration=auto_cleanup_duration,
     )
 
 
@@ -637,12 +921,16 @@ def get_volume_provisioning_data(
     volume_id: str = "vol-1234",
     size_gb: int = 100,
     availability_zone: Optional[str] = None,
+    price: Optional[float] = 1.0,
     backend_data: Optional[str] = None,
+    backend: Optional[BackendType] = None,
 ) -> VolumeProvisioningData:
     return VolumeProvisioningData(
+        backend=backend,
         volume_id=volume_id,
         size_gb=size_gb,
         availability_zone=availability_zone,
+        price=price,
         backend_data=backend_data,
     )
 
@@ -726,6 +1014,38 @@ async def create_job_metrics_point(
     return jmp
 
 
+async def create_job_prometheus_metrics(
+    session: AsyncSession,
+    job: JobModel,
+    collected_at: datetime = datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
+    text: str = "# Prometheus metrics\n",
+):
+    metrics = JobPrometheusMetrics(
+        job_id=job.id,
+        collected_at=collected_at,
+        text=text,
+    )
+    session.add(metrics)
+    await session.commit()
+    return metrics
+
+
+async def create_secret(
+    session: AsyncSession,
+    project: ProjectModel,
+    name: str,
+    value: str,
+):
+    secret_model = SecretModel(
+        project=project,
+        name=name,
+        value=DecryptedString(plaintext=value),
+    )
+    session.add(secret_model)
+    await session.commit()
+    return secret_model
+
+
 def get_private_key_string() -> str:
     return """
 -----BEGIN RSA PRIVATE KEY-----
@@ -798,3 +1118,20 @@ class AsyncContextManager:
 
     async def __aexit__(self, exc_type, exc, traceback):
         pass
+
+
+class ComputeMockSpec(
+    Compute,
+    ComputeWithCreateInstanceSupport,
+    ComputeWithMultinodeSupport,
+    ComputeWithReservationSupport,
+    ComputeWithPlacementGroupSupport,
+    ComputeWithGatewaySupport,
+    ComputeWithPrivateGatewaySupport,
+    ComputeWithVolumeSupport,
+):
+    """
+    Can be used to create Compute mocks that pass all `isinstance()` asserts.
+    """
+
+    pass
