@@ -1,12 +1,25 @@
+import asyncio
+import enum
 import itertools
 import re
 import time
 from collections.abc import Callable
 from datetime import datetime, timedelta, timezone
+from functools import partial
 from pathlib import Path
 from typing import Any, Iterable, List, Optional, TypeVar
 from urllib.parse import urlparse
 
+from typing_extensions import ParamSpec
+
+P = ParamSpec("P")
+R = TypeVar("R")
+
+
+async def run_async(func: Callable[P, R], *args: P.args, **kwargs: P.kwargs) -> R:
+    func_with_args = partial(func, *args, **kwargs)
+    return await asyncio.get_running_loop().run_in_executor(None, func_with_args)
+
 
 def get_dstack_dir() -> Path:
     return Path.joinpath(Path.home(), ".dstack")
@@ -55,9 +68,15 @@ def pretty_date(time: datetime) -> str:
     if diff.days < 7:
         return str(diff.days) + " days ago"
     if diff.days < 31:
-        return str(round(diff.days / 7)) + " weeks ago"
+        weeks = round(diff.days / 7)
+        if weeks == 1:
+            return str(weeks) + " week ago"
+        return str(weeks) + " weeks ago"
     if diff.days < 365:
-        return str(round(diff.days / 30)) + " months ago"
+        months = round(diff.days / 30)
+        if months == 1:
+            return str(months) + " month ago"
+        return str(months) + " months ago"
     years = round(diff.days / 365)
     if years == 1:
         return str(years) + " year ago"
@@ -65,6 +84,8 @@ def pretty_date(time: datetime) -> str:
 
 
 def pretty_resources(
+    *,
+    cpu_arch: Optional[Any] = None,
     cpus: Optional[Any] = None,
     memory: Optional[Any] = None,
     gpu_count: Optional[Any] = None,
@@ -92,25 +113,35 @@ def pretty_resources(
     """
     parts = []
     if cpus is not None:
-        parts.append(f"{cpus}xCPU")
+        cpu_arch_lower: Optional[str] = None
+        if isinstance(cpu_arch, enum.Enum):
+            cpu_arch_lower = str(cpu_arch.value).lower()
+        elif isinstance(cpu_arch, str):
+            cpu_arch_lower = cpu_arch.lower()
+        if cpu_arch_lower == "arm":
+            cpu_arch_prefix = "arm:"
+        else:
+            cpu_arch_prefix = ""
+        parts.append(f"cpu={cpu_arch_prefix}{cpus}")
     if memory is not None:
-        parts.append(f"{memory}")
+        parts.append(f"mem={memory}")
+    if disk_size:
+        parts.append(f"disk={disk_size}")
     if gpu_count:
         gpu_parts = []
+        gpu_parts.append(f"{gpu_name or 'gpu'}")
         if gpu_memory is not None:
             gpu_parts.append(f"{gpu_memory}")
+        if gpu_count is not None:
+            gpu_parts.append(f"{gpu_count}")
         if total_gpu_memory is not None:
-            gpu_parts.append(f"total {total_gpu_memory}")
+            gpu_parts.append(f"{total_gpu_memory}")
         if compute_capability is not None:
             gpu_parts.append(f"{compute_capability}")
 
-        gpu = f"{gpu_count}x{gpu_name or 'GPU'}"
-        if gpu_parts:
-            gpu += f" ({', '.join(gpu_parts)})"
+        gpu = ":".join(gpu_parts)
         parts.append(gpu)
-    if disk_size:
-        parts.append(f"{disk_size} (disk)")
-    return ", ".join(parts)
+    return " ".join(parts)
 
 
 def since(timestamp: str) -> datetime:
@@ -145,22 +176,38 @@ def parse_pretty_duration(duration: str) -> int:
     return amount * multiplier
 
 
+DURATION_UNITS_DESC = [
+    ("w", 7 * 24 * 3600),
+    ("d", 24 * 3600),
+    ("h", 3600),
+    ("m", 60),
+    ("s", 1),
+]
+
+
 def format_pretty_duration(seconds: int) -> str:
+    if seconds == 0:
+        return "0s"
     if seconds < 0:
         raise ValueError("Seconds cannot be negative")
-    units = [
-        ("w", 7 * 24 * 3600),
-        ("d", 24 * 3600),
-        ("h", 3600),
-        ("m", 60),
-        ("s", 1),
-    ]
-    for unit, multiplier in units:
+    for unit, multiplier in DURATION_UNITS_DESC:
         if seconds % multiplier == 0:
             return f"{seconds // multiplier}{unit}"
     return f"{seconds}s"  # Fallback to seconds if no larger unit fits perfectly
 
 
+def format_duration_multiunit(seconds: int) -> str:
+    """90 -> 1m 30s, 4545 -> 1h 15m 45s, etc"""
+    if seconds < 0:
+        raise ValueError("Seconds cannot be negative")
+    result = ""
+    for unit, multiplier in DURATION_UNITS_DESC:
+        if unit_value := seconds // multiplier:
+            result += f" {unit_value}{unit}"
+            seconds -= unit_value * multiplier
+    return result.lstrip() or "0s"
+
+
 def sizeof_fmt(num, suffix="B"):
     for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
         if abs(num) < 1024.0:
@@ -175,28 +222,22 @@ def remove_prefix(text: str, prefix: str) -> str:
     return text
 
 
-T = TypeVar("T")
-
-
-def split_chunks(iterable: Iterable[T], chunk_size: int) -> Iterable[List[T]]:
+def has_duplicates(iterable: Iterable[Any]) -> bool:
     """
-    Splits an iterable into chunks of at most `chunk_size` items.
+    Checks if there are any duplicate items in the given iterable.
 
-    >>> list(split_chunks([1, 2, 3, 4, 5], 2))
-    [[1, 2], [3, 4], [5]]
+    O(n^2) implementation, but works with iterables with unhashable items.
+    For iterables with hashable items, prefer len(set(iterable)) != len(iterable).
     """
-
-    if chunk_size < 1:
-        raise ValueError(f"chunk_size should be a positive integer, not {chunk_size}")
-
-    chunk = []
+    seen = []
     for item in iterable:
-        chunk.append(item)
-        if len(chunk) == chunk_size:
-            yield chunk
-            chunk = []
-    if chunk:
-        yield chunk
+        if item in seen:
+            return True
+        seen.append(item)
+    return False
+
+
+T = TypeVar("T")
 
 
 MEMORY_UNITS = {
@@ -236,7 +277,17 @@ def get_or_error(v: Optional[T]) -> T:
     return v
 
 
+# TODO: drop after dropping Python 3.11
 def batched(seq: Iterable[T], n: int) -> Iterable[List[T]]:
+    """
+    Roughly equivalent to itertools.batched from Python 3.12.
+
+    >>> list(batched([1, 2, 3, 4, 5], 2))
+    [[1, 2], [3, 4], [5]]
+    """
+
+    if n < 1:
+        raise ValueError(f"n should be a positive integer, not {n}")
     it = iter(seq)
     return iter(lambda: list(itertools.islice(it, n)), [])
 
@@ -267,3 +318,7 @@ def make_proxy_url(server_url: str, proxy_url: str) -> str:
         path=concat_url_path(server.path, proxy.path),
     )
     return proxy.geturl()
+
+
+def list_enum_values_for_annotation(enum_class: type[enum.Enum]) -> str:
+    return ", ".join(f"`{e.value}`" for e in enum_class)
