@@ -55,7 +55,7 @@ class RepoCollection:
         Once the repo is initialized, you can pass the repo object to the run:
 
         ```python
-        run = client.runs.submit(
+        run = client.runs.apply_configuration(
             configuration=...,
             repo=repo,
         )
@@ -78,25 +78,6 @@ class RepoCollection:
                 raise ConfigurationError(*e.args)
         self._api_client.repos.init(self._project, repo.repo_id, repo.get_repo_info(), creds)
 
-    def is_initialized(
-        self,
-        repo: Repo,
-    ) -> bool:
-        # """
-        # Checks if the remote repo is initialized in the project
-        #
-        # Args:
-        #     repo: repo to check
-        #
-        # Returns:
-        #     repo is initialized
-        # """
-        try:
-            self._api_client.repos.get(self._project, repo.repo_id, include_creds=False)
-            return True
-        except ResourceNotExistsError:
-            return False
-
     def load(
         self,
         repo_dir: PathLike,
@@ -105,43 +86,45 @@ class RepoCollection:
         git_identity_file: Optional[PathLike] = None,
         oauth_token: Optional[str] = None,
     ) -> Union[RemoteRepo, LocalRepo]:
-        # """
-        # Loads the repo from the local directory using global config
-        #
-        # Args:
-        #     repo_dir: repo root directory
-        #     local: do not try to load `RemoteRepo` first
-        #     init: initialize the repo if it's not initialized
-        #     git_identity_file: path to an SSH private key to access the remote repo
-        #     oauth_token: GitHub OAuth token to access the remote repo
-        #
-        # Raises:
-        #     ConfigurationError: if the repo is not initialized and `init` is `False`
-        #
-        # Returns:
-        #     repo: initialized repo
-        # """
+        """
+        Loads the repo from the local directory using global config
+
+        Args:
+            repo_dir: Repo root directory.
+            local: Do not try to load `RemoteRepo` first.
+            init: Initialize the repo if it's not initialized.
+            git_identity_file: Path to an SSH private key to access the remote repo.
+            oauth_token: GitHub OAuth token to access the remote repo.
+
+        Raises:
+            ConfigurationError: If the repo is not initialized and `init` is `False`.
+
+        Returns:
+            repo: Initialized repo.
+        """
         config = ConfigManager()
         if not init:
             logger.debug("Loading repo config")
             repo_config = config.get_repo_config(repo_dir)
             if repo_config is None:
                 raise ConfigurationError(
-                    "The repo is not initialized. Run `dstack init` to initialize the repo."
+                    "The repo is not initialized."
+                    " Run `dstack init` to initialize the current directory as a repo or specify `--repo`."
                 )
             repo = load_repo(repo_config)
             try:
                 self._api_client.repos.get(self._project, repo.repo_id, include_creds=False)
             except ResourceNotExistsError:
                 raise ConfigurationError(
-                    "The repo is not initialized. Run `dstack init` to initialize the repo."
+                    "The repo is not initialized."
+                    " Run `dstack init` to initialize the current directory as a repo or specify `--repo`."
                 )
         else:
             logger.debug("Initializing repo")
             repo = LocalRepo(repo_dir=repo_dir)  # default
             if not local:
                 try:
-                    repo = RemoteRepo(local_repo_dir=repo_dir)
+                    repo = RemoteRepo.from_dir(repo_dir)
                 except InvalidGitRepositoryError:
                     pass  # use default
             self.init(repo, git_identity_file, oauth_token)
@@ -153,6 +136,25 @@ class RepoCollection:
             )
         return repo
 
+    def is_initialized(
+        self,
+        repo: Repo,
+    ) -> bool:
+        """
+        Checks if the remote repo is initialized in the project
+
+        Args:
+            repo: The repo to check.
+
+        Returns:
+            Whether the repo is initialized or not.
+        """
+        try:
+            self._api_client.repos.get(self._project, repo.repo_id, include_creds=False)
+            return True
+        except ResourceNotExistsError:
+            return False
+
 
 def get_ssh_keypair(key_path: Optional[PathLike], dstack_key_path: Path) -> str:
     """Returns a path to the private key"""
