@@ -4,37 +4,45 @@ import tempfile
 import threading
 import time
 from abc import ABC
+from collections.abc import Iterator
+from contextlib import contextmanager
 from copy import copy
 from datetime import datetime
 from pathlib import Path
-from typing import Dict, Iterable, List, Optional, Union
+from typing import BinaryIO, Dict, Iterable, List, Optional, Union
 from urllib.parse import urlparse
 
 from websocket import WebSocketApp
 
 import dstack.api as api
+from dstack._internal.core.consts import DSTACK_RUNNER_HTTP_PORT, DSTACK_RUNNER_SSH_PORT
 from dstack._internal.core.errors import ClientError, ConfigurationError, ResourceNotExistsError
 from dstack._internal.core.models.backends.base import BackendType
-from dstack._internal.core.models.common import ApplyAction
-from dstack._internal.core.models.configurations import AnyRunConfiguration, PortMapping
-from dstack._internal.core.models.pools import Instance
+from dstack._internal.core.models.configurations import (
+    AnyRunConfiguration,
+    PortMapping,
+    ServiceConfiguration,
+)
+from dstack._internal.core.models.files import FileArchiveMapping, FilePathMapping
 from dstack._internal.core.models.profiles import (
     CreationPolicy,
     Profile,
     ProfileRetryPolicy,
     SpotPolicy,
     TerminationPolicy,
+    UtilizationPolicy,
 )
 from dstack._internal.core.models.repos.base import Repo
+from dstack._internal.core.models.repos.virtual import VirtualRepo
 from dstack._internal.core.models.resources import ResourcesSpec
 from dstack._internal.core.models.runs import (
     Job,
     JobSpec,
-    PoolInstanceOffers,
-    Requirements,
+    JobStatus,
     RunPlan,
     RunSpec,
     RunStatus,
+    get_service_port,
 )
 from dstack._internal.core.models.runs import Run as RunModel
 from dstack._internal.core.services.logs import URLReplacer
@@ -42,6 +50,7 @@ from dstack._internal.core.services.ssh.attach import SSHAttach
 from dstack._internal.core.services.ssh.ports import PortsLock
 from dstack._internal.server.schemas.logs import PollLogsRequest
 from dstack._internal.utils.common import get_or_error, make_proxy_url
+from dstack._internal.utils.files import create_file_archive
 from dstack._internal.utils.logging import get_logger
 from dstack._internal.utils.path import PathLike, path_in_dir
 from dstack.api.server import APIClient
@@ -135,7 +144,7 @@ class Run(ABC):
                 q.put(_done)
 
         ws = WebSocketApp(
-            f"ws://localhost:{self.ports[10999]}/logs_ws",
+            f"ws://localhost:{self.ports[DSTACK_RUNNER_HTTP_PORT]}/logs_ws",
             on_open=lambda _: logger.debug("WebSocket logs are connected to %s", self.name),
             on_close=lambda _, status_code, msg: logger.debug(
                 "WebSocket logs are disconnected. status_code: %s; message: %s",
@@ -159,7 +168,7 @@ class Run(ABC):
                 service_port = 443 if secure else 80
             ports = {
                 **ports,
-                self._run.run_spec.configuration.port.container_port: service_port,
+                get_or_error(get_or_error(self._ssh_attach).service_port): service_port,
             }
             path_prefix = url.path
         replace_urls = URLReplacer(
@@ -185,18 +194,18 @@ class Run(ABC):
         self,
         start_time: Optional[datetime] = None,
         diagnose: bool = False,
-        replica_num: int = 0,
+        replica_num: Optional[int] = None,
         job_num: int = 0,
     ) -> Iterable[bytes]:
         """
-        Iterate through run's log messages
+        Iterate through run's log messages.
 
         Args:
-            start_time: minimal log timestamp
-            diagnose: return runner logs if `True`
+            start_time: Minimal log timestamp.
+            diagnose: Return runner logs if `True`.
 
         Yields:
-            log messages
+            Log messages.
         """
         if diagnose is False and self._ssh_attach is not None:
             yield from self._attached_logs()
@@ -204,39 +213,40 @@ class Run(ABC):
             job = self._find_job(replica_num=replica_num, job_num=job_num)
             if job is None:
                 return []
-            next_start_time = start_time
+            next_token = None
             while True:
                 resp = self._api_client.logs.poll(
                     project_name=self._project,
                     body=PollLogsRequest(
                         run_name=self.name,
                         job_submission_id=job.job_submissions[-1].id,
-                        start_time=next_start_time,
+                        start_time=start_time,
                         end_time=None,
                         descending=False,
-                        limit=100,
+                        limit=1000,
                         diagnose=diagnose,
+                        next_token=next_token,
                     ),
                 )
-                if len(resp.logs) == 0:
-                    return []
                 for log in resp.logs:
                     yield base64.b64decode(log.message)
-                next_start_time = resp.logs[-1].timestamp
+                next_token = resp.next_token
+                if next_token is None:
+                    break
 
     def refresh(self):
         """
-        Get up-to-date run info
+        Get up-to-date run info.
         """
         self._run = self._api_client.runs.get(self._project, self._run.run_spec.run_name)
         logger.debug("Refreshed run %s: %s", self.name, self.status)
 
     def stop(self, abort: bool = False):
         """
-        Terminate the instance and detach
+        Terminate the instance and detach.
 
         Args:
-            abort: gracefully stop the run if `False`
+            abort: Gracefully stop the run if `False`.
         """
         self._api_client.runs.stop(self._project, [self.name], abort)
         logger.debug("%s run %s", "Aborted" if abort else "Stopped", self.name)
@@ -247,14 +257,15 @@ class Run(ABC):
         ssh_identity_file: Optional[PathLike] = None,
         bind_address: Optional[str] = None,
         ports_overrides: Optional[List[PortMapping]] = None,
-        replica_num: int = 0,
+        replica_num: Optional[int] = None,
         job_num: int = 0,
     ) -> bool:
         """
         Establish an SSH tunnel to the instance and update SSH config
 
         Args:
-            ssh_identity_file: SSH keypair to access instances
+            ssh_identity_file: SSH keypair to access instances.
+            replica_num: replica_num or None to attach to any running replica.
 
         Raises:
             dstack.api.PortUsedError: If ports are in use or the run is attached by another process.
@@ -266,7 +277,9 @@ class Run(ABC):
 
         job = self._find_job(replica_num=replica_num, job_num=job_num)
         if job is None:
-            raise ClientError(f"Failed to find replica={replica_num} job={job_num}")
+            replica_repr = replica_num if replica_num is not None else "<any running>"
+            raise ClientError(f"Failed to find replica={replica_repr} job={job_num}")
+        replica_num = job.job_spec.replica_num
 
         name = self.name
         if replica_num != 0 or job_num != 0:
@@ -293,7 +306,8 @@ class Run(ABC):
 
             # Reload job
             job = get_or_error(self._find_job(replica_num=replica_num, job_num=job_num))
-            provisioning_data = job.job_submissions[-1].job_provisioning_data
+            latest_job_submission = job.job_submissions[-1]
+            provisioning_data = latest_job_submission.job_provisioning_data
             if provisioning_data is None:
                 raise ClientError("Failed to attach. The run is not provisioned yet.")
 
@@ -317,15 +331,34 @@ class Run(ABC):
                     self._ports_lock.dict(),
                 )
 
+            container_ssh_port = DSTACK_RUNNER_SSH_PORT
+            runtime_data = latest_job_submission.job_runtime_data
+            if runtime_data is not None and runtime_data.ports is not None:
+                container_ssh_port = runtime_data.ports.get(container_ssh_port, container_ssh_port)
+
+            # TODO: get login name from runner in case it's not specified in the run configuration
+            # (i.e. the default image user is used, and it is not root)
+            if job.job_spec.user is not None and job.job_spec.user.username is not None:
+                container_user = job.job_spec.user.username
+            else:
+                container_user = "root"
+
+            service_port = None
+            if isinstance(self._run.run_spec.configuration, ServiceConfiguration):
+                service_port = get_service_port(job.job_spec, self._run.run_spec.configuration)
+
             self._ssh_attach = SSHAttach(
                 hostname=provisioning_data.hostname,
                 ssh_port=provisioning_data.ssh_port,
+                container_ssh_port=container_ssh_port,
                 user=provisioning_data.username,
+                container_user=container_user,
                 id_rsa_path=ssh_identity_file,
                 ports_lock=self._ports_lock,
                 run_name=name,
                 dockerized=provisioning_data.dockerized,
                 ssh_proxy=provisioning_data.ssh_proxy,
+                service_port=service_port,
                 local_backend=provisioning_data.backend == BackendType.LOCAL,
                 bind_address=bind_address,
             )
@@ -344,9 +377,14 @@ class Run(ABC):
             self._ssh_attach.detach()
             self._ssh_attach = None
 
-    def _find_job(self, replica_num: int, job_num: int) -> Optional[Job]:
+    def _find_job(self, replica_num: Optional[int], job_num: int) -> Optional[Job]:
         for j in self._run.jobs:
-            if j.job_spec.replica_num == replica_num and j.job_spec.job_num == job_num:
+            if (
+                replica_num is not None
+                and j.job_spec.replica_num == replica_num
+                or replica_num is None
+                and j.job_submissions[-1].status == JobStatus.RUNNING
+            ) and j.job_spec.job_num == job_num:
                 return j
         return None
 
@@ -376,7 +414,7 @@ class ServiceModel:
 
 class RunCollection:
     """
-    Operations with runs
+    Operations with runs.
     """
 
     def __init__(
@@ -389,6 +427,142 @@ class RunCollection:
         self._project = project
         self._client = client
 
+    def get_run_plan(
+        self,
+        configuration: AnyRunConfiguration,
+        repo: Optional[Repo] = None,
+        profile: Optional[Profile] = None,
+        configuration_path: Optional[str] = None,
+    ) -> RunPlan:
+        """
+        Get a run plan.
+        Use this method to see the run plan before applying the cofiguration.
+
+        Args:
+            configuration (Union[Task, Service, DevEnvironment]): The run configuration.
+            repo (Union[LocalRepo, RemoteRepo, VirtualRepo, None]):
+                The repo to use for the run. Pass `None` if repo is not needed.
+            profile: The profile to use for the run.
+            configuration_path: The path to the configuration file. Omit if the configuration is not loaded from a file.
+
+        Returns:
+            Run plan.
+        """
+        if repo is None:
+            repo = VirtualRepo()
+            repo_code_hash = None
+        else:
+            with _prepare_code_file(repo) as (_, repo_code_hash):
+                pass
+
+        run_spec = RunSpec(
+            run_name=configuration.name,
+            repo_id=repo.repo_id,
+            repo_data=repo.run_repo_data,
+            repo_code_hash=repo_code_hash,
+            working_dir=configuration.working_dir,
+            configuration_path=configuration_path,
+            configuration=configuration,
+            profile=profile,
+            ssh_key_pub=Path(self._client.ssh_identity_file + ".pub").read_text().strip(),
+        )
+        logger.debug("Getting run plan")
+        run_plan = self._api_client.runs.get_plan(self._project, run_spec)
+        return run_plan
+
+    def apply_plan(
+        self,
+        run_plan: RunPlan,
+        repo: Optional[Repo] = None,
+        reserve_ports: bool = True,
+    ) -> Run:
+        """
+        Apply the run plan.
+        Use this method to apply run plans returned by `get_run_plan`.
+
+        Args:
+            run_plan: The result of `get_run_plan` call.
+            repo (Union[LocalRepo, RemoteRepo, VirtualRepo, None]):
+                The repo to use for the run. Should be the same repo that is passed to `get_run_plan`.
+            reserve_ports: Reserve local ports before applying. Use if you'll attach to the run.
+
+        Returns:
+            Submitted run.
+        """
+        ports_lock = None
+        if reserve_ports:
+            # TODO handle multiple jobs
+            ports_lock = _reserve_ports(run_plan.job_plans[0].job_spec)
+
+        run_spec = run_plan.run_spec
+        configuration = run_spec.configuration
+
+        self._validate_configuration_files(configuration, run_spec.configuration_path)
+        for file_mapping in configuration.files:
+            assert isinstance(file_mapping, FilePathMapping)
+            with tempfile.TemporaryFile("w+b") as fp:
+                try:
+                    archive_hash = create_file_archive(file_mapping.local_path, fp)
+                except OSError as e:
+                    raise ClientError(f"failed to archive '{file_mapping.local_path}': {e}") from e
+                fp.seek(0)
+                archive = self._api_client.files.upload_archive(hash=archive_hash, fp=fp)
+            run_spec.file_archives.append(
+                FileArchiveMapping(id=archive.id, path=file_mapping.path)
+            )
+
+        if repo is None:
+            repo = VirtualRepo()
+        else:
+            # Do not upload the diff without a repo (a default virtual repo)
+            # since upload_code() requires a repo to be initialized.
+            with _prepare_code_file(repo) as (fp, repo_code_hash):
+                self._api_client.repos.upload_code(
+                    project_name=self._project,
+                    repo_id=repo.repo_id,
+                    code_hash=repo_code_hash,
+                    fp=fp,
+                )
+
+        run = self._api_client.runs.apply_plan(self._project, run_plan)
+        return self._model_to_submitted_run(run, ports_lock)
+
+    def apply_configuration(
+        self,
+        configuration: AnyRunConfiguration,
+        repo: Optional[Repo] = None,
+        profile: Optional[Profile] = None,
+        configuration_path: Optional[str] = None,
+        reserve_ports: bool = True,
+    ) -> Run:
+        """
+        Apply the run configuration.
+        Use this method to apply configurations without getting a run plan first.
+
+        Args:
+            configuration (Union[Task, Service, DevEnvironment]): The run configuration.
+            repo (Union[LocalRepo, RemoteRepo, VirtualRepo, None]):
+                The repo to use for the run. Pass `None` if repo is not needed.
+            profile: The profile to use for the run.
+            configuration_path: The path to the configuration file. Omit if the configuration is not loaded from a file.
+            reserve_ports: Reserve local ports before applying. Use if you'll attach to the run.
+
+        Returns:
+            Submitted run.
+        """
+        run_plan = self.get_run_plan(
+            configuration=configuration,
+            repo=repo,
+            profile=profile,
+            configuration_path=configuration_path,
+        )
+        run = self.apply_plan(
+            run_plan=run_plan,
+            repo=repo,
+            reserve_ports=reserve_ports,
+        )
+        return run
+
     def submit(
         self,
         configuration: AnyRunConfiguration,
@@ -406,31 +580,30 @@ class RunCollection:
         run_name: Optional[str] = None,
         reserve_ports: bool = True,
     ) -> Run:
-        """
-        Submit a run
+        # """
+        # Submit a run
 
-        Args:
-            configuration (Union[Task, Service]): A run configuration.
-            configuration_path: The path to the configuration file, relative to the root directory of the repo.
-            repo (Union[LocalRepo, RemoteRepo, VirtualRepo]): A repo to mount to the run.
-            backends: A list of allowed backend for provisioning.
-            regions: A list of cloud regions for provisioning.
-            resources: The requirements to run the configuration. Overrides the configuration's resources.
-            spot_policy: A spot policy for provisioning.
-            retry_policy (RetryPolicy): A retry policy.
-            max_duration: The max instance running duration in seconds.
-            max_price: The max instance price in dollars per hour for provisioning.
-            working_dir: A working directory relative to the repo root directory
-            run_name: A desired name of the run. Must be unique in the project. If not specified, a random name is assigned.
-            reserve_ports: Whether local ports should be reserved in advance.
+        # Args:
+        #     configuration (Union[Task, Service]): A run configuration.
+        #     configuration_path: The path to the configuration file, relative to the root directory of the repo.
+        #     repo (Union[LocalRepo, RemoteRepo, VirtualRepo]): A repo to mount to the run.
+        #     backends: A list of allowed backend for provisioning.
+        #     regions: A list of cloud regions for provisioning.
+        #     resources: The requirements to run the configuration. Overrides the configuration's resources.
+        #     spot_policy: A spot policy for provisioning.
+        #     retry_policy (RetryPolicy): A retry policy.
+        #     max_duration: The max instance running duration in seconds.
+        #     max_price: The max instance price in dollars per hour for provisioning.
+        #     working_dir: A working directory relative to the repo root directory
+        #     run_name: A desired name of the run. Must be unique in the project. If not specified, a random name is assigned.
+        #     reserve_ports: Whether local ports should be reserved in advance.
 
-        Returns:
-            submitted run
-        """
+        # Returns:
+        #     Submitted run.
+        # """
+        logger.warning("The submit() method is deprecated in favor of apply_configuration().")
         if repo is None:
-            repo = configuration.get_repo()
-            if repo is None:
-                raise ConfigurationError("Repo is required for this type of configuration")
+            repo = VirtualRepo()
             # TODO: Add Git credentials to RemoteRepo and if they are set, pass them here to RepoCollection.init
             self._client.repos.init(repo)
 
@@ -451,23 +624,21 @@ class RunCollection:
         )
         return self.exec_plan(run_plan, repo, reserve_ports=reserve_ports)
 
-    def get_offers(self, profile: Profile, requirements: Requirements) -> PoolInstanceOffers:
-        return self._api_client.runs.get_offers(self._project, profile, requirements)
-
-    def create_instance(self, profile: Profile, requirements: Requirements) -> Instance:
-        return self._api_client.runs.create_instance(self._project, profile, requirements)
-
+    # Deprecated in favor of get_run_plan()
     def get_plan(
         self,
         configuration: AnyRunConfiguration,
         repo: Optional[Repo] = None,
         configuration_path: Optional[str] = None,
+        # Unused profile args are deprecated and removed but
+        # kept for signature backward compatibility.
         backends: Optional[List[BackendType]] = None,
         regions: Optional[List[str]] = None,
         instance_types: Optional[List[str]] = None,
         resources: Optional[ResourcesSpec] = None,
         spot_policy: Optional[SpotPolicy] = None,
         retry_policy: Optional[ProfileRetryPolicy] = None,
+        utilization_policy: Optional[UtilizationPolicy] = None,
         max_duration: Optional[Union[int, str]] = None,
         max_price: Optional[float] = None,
         working_dir: Optional[str] = None,
@@ -477,6 +648,9 @@ class RunCollection:
         creation_policy: Optional[CreationPolicy] = None,
         termination_policy: Optional[TerminationPolicy] = None,
         termination_policy_idle: Optional[Union[str, int]] = None,
+        reservation: Optional[str] = None,
+        idle_duration: Optional[Union[str, int]] = None,
+        stop_duration: Optional[Union[str, int]] = None,
     ) -> RunPlan:
         # """
         # Get run plan. Same arguments as `submit`
@@ -484,11 +658,13 @@ class RunCollection:
         # Returns:
         #     run plan
         # """
-
+        logger.warning("The get_plan() method is deprecated in favor of get_run_plan().")
         if repo is None:
-            repo = configuration.get_repo()
-            if repo is None:
-                raise ConfigurationError("Repo is required for this type of configuration")
+            repo = VirtualRepo()
+            repo_code_hash = None
+        else:
+            with _prepare_code_file(repo) as (_, repo_code_hash):
+                pass
 
         if working_dir is None:
             working_dir = "."
@@ -505,27 +681,27 @@ class RunCollection:
             configuration = configuration.copy(deep=True)
             configuration.resources = resources
 
+        # TODO: [Andrey] "(python") looks as a hack
         profile = Profile(
             name="(python)",
             backends=backends,
             regions=regions,
             instance_types=instance_types,
+            reservation=reservation,
             spot_policy=spot_policy,
             retry=None,
-            retry_policy=retry_policy,
+            utilization_policy=utilization_policy,
             max_duration=max_duration,
+            stop_duration=stop_duration,
             max_price=max_price,
-            pool_name=pool_name,
-            instance_name=instance_name,
             creation_policy=creation_policy,
-            termination_policy=termination_policy,
-            termination_idle_time=termination_policy_idle,
+            idle_duration=idle_duration,
         )
         run_spec = RunSpec(
             run_name=run_name,
             repo_id=repo.repo_id,
             repo_data=repo.run_repo_data,
-            repo_code_hash=None,  # `exec_plan` will fill it
+            repo_code_hash=repo_code_hash,
             working_dir=working_dir,
             configuration_path=configuration_path,
             configuration=configuration,
@@ -550,70 +726,57 @@ class RunCollection:
         reserve_ports: bool = True,
     ) -> Run:
         # """
-        # Execute run plan
-        #
+        # Execute the run plan.
+
         # Args:
-        #     run_plan: result of `get_plan` call
-        #     repo: repo to use for the run
-        #     reserve_ports: reserve local ports before submit
-        #
+        #     run_plan: Result of `get_run_plan` call.
+        #     repo: Repo to use for the run.
+        #     reserve_ports: Reserve local ports before submit.
+
         # Returns:
-        #     submitted run
+        #     Submitted run.
         # """
-        ports_lock = None
-        if reserve_ports:
-            # TODO handle multiple jobs
-            ports_lock = _reserve_ports(run_plan.job_plans[0].job_spec)
+        logger.warning("The exec_plan() method is deprecated in favor of apply_plan().")
+        return self.apply_plan(run_plan=run_plan, repo=repo, reserve_ports=reserve_ports)
 
-        with tempfile.TemporaryFile("w+b") as fp:
-            run_plan.run_spec.repo_code_hash = repo.write_code_file(fp)
-            fp.seek(0)
-            self._api_client.repos.upload_code(
-                self._project, repo.repo_id, run_plan.run_spec.repo_code_hash, fp
-            )
-        # Calling submit when action is CREATE since apply_plan is not backward-compatible.
-        # Otherwise, apply_plan can replace submit, i.e. it creates the run if it does not exist.
-        # TODO: Remove in 0.19
-        if run_plan.action == ApplyAction.UPDATE:
-            run = self._api_client.runs.apply_plan(self._project, run_plan)
-        else:
-            run = self._api_client.runs.submit(self._project, run_plan.run_spec)
-        return self._model_to_submitted_run(run, ports_lock)
-
-    def list(self, all: bool = False) -> List[Run]:
+    def list(self, all: bool = False, limit: Optional[int] = None) -> List[Run]:
         """
-        List runs
+        List runs.
 
         Args:
-            all: show all runs (active and finished) if `True`
+            all: Show all runs (active and finished) if `True`.
+            limit: Limit the number of runs to return. Must be less than 100.
 
         Returns:
-            list of runs
+            List of runs.
         """
         # Return only one page of latest runs (<=100). Returning all the pages may be costly.
         # TODO: Consider introducing `since` filter with a reasonable default.
-        only_active = not all
+        only_active = not all and limit is None
         runs = self._api_client.runs.list(
             project_name=self._project,
             repo_id=None,
             only_active=only_active,
+            limit=limit or 100,
+            # TODO: Pass job_submissions_limit=1 in 0.20
         )
         if only_active and len(runs) == 0:
             runs = self._api_client.runs.list(
                 project_name=self._project,
                 repo_id=None,
-            )[:1]
+                limit=1,
+            )
         return [self._model_to_run(run) for run in runs]
 
     def get(self, run_name: str) -> Optional[Run]:
         """
-        Get run by run name
+        Get run by run name.
 
         Args:
-            run_name: run name
+            run_name: Run name.
 
         Returns:
-            The run or `None` if not found
+            The run or `None` if not found.
         """
         try:
             run = self._api_client.runs.get(self._project, run_name)
@@ -638,6 +801,30 @@ class RunCollection:
             ports_lock,
         )
 
+    def _validate_configuration_files(
+        self, configuration: AnyRunConfiguration, configuration_path: Optional[PathLike]
+    ) -> None:
+        """
+        Expands, normalizes and validates local paths specified in
+        the `files` configuration property.
+        """
+        base_dir: Optional[Path] = None
+        if configuration_path is not None:
+            base_dir = Path(configuration_path).expanduser().resolve().parent
+        for file_mapping in configuration.files:
+            assert isinstance(file_mapping, FilePathMapping)
+            path = Path(file_mapping.local_path).expanduser()
+            if not path.is_absolute():
+                if base_dir is None:
+                    raise ConfigurationError(
+                        f"Path '{path}' is relative but `configuration_path` is not provided"
+                    )
+                else:
+                    path = base_dir / path
+            if not path.exists():
+                raise ConfigurationError(f"Path '{path}' specified in `files` does not exist")
+            file_mapping.local_path = str(path)
+
 
 def _reserve_ports(
     job_spec: JobSpec,
@@ -645,7 +832,7 @@ def _reserve_ports(
 ) -> PortsLock:
     if ports_overrides is None:
         ports_overrides = []
-    ports = {10999: 0}  # Runner API
+    ports = {DSTACK_RUNNER_HTTP_PORT: 0}
     if job_spec.app_specs:
         for app in job_spec.app_specs:
             ports[app.port] = app.map_to_port or 0
@@ -657,3 +844,11 @@ def _reserve_ports(
         ports[port_override.container_port] = port_override.local_port or 0
     logger.debug("Reserving ports: %s", ports)
     return PortsLock(ports).acquire()
+
+
+@contextmanager
+def _prepare_code_file(repo: Repo) -> Iterator[tuple[BinaryIO, str]]:
+    with tempfile.TemporaryFile("w+b") as fp:
+        repo_code_hash = repo.write_code_file(fp)
+        fp.seek(0)
+        yield fp, repo_code_hash
