@@ -6,14 +6,19 @@ from typing import Dict, List, Optional, Type
 import requests
 
 from dstack import version
-from dstack._internal.core.errors import ClientError, ServerClientError
+from dstack._internal.core.errors import (
+    ClientError,
+    MethodNotAllowedError,
+    ServerClientError,
+    URLNotFoundError,
+)
 from dstack._internal.utils.logging import get_logger
 from dstack.api.server._backends import BackendsAPIClient
+from dstack.api.server._files import FilesAPIClient
 from dstack.api.server._fleets import FleetsAPIClient
 from dstack.api.server._gateways import GatewaysAPIClient
 from dstack.api.server._logs import LogsAPIClient
 from dstack.api.server._metrics import MetricsAPIClient
-from dstack.api.server._pools import PoolAPIClient
 from dstack.api.server._projects import ProjectsAPIClient
 from dstack.api.server._repos import ReposAPIClient
 from dstack.api.server._runs import RunsAPIClient
@@ -30,23 +35,27 @@ _RETRY_INTERVAL = 1
 
 class APIClient:
     """
-    Low-level API client for interacting with dstack server. Implements all API endpoints
+    Low-level API client for interacting with the `dstack` server.
+    Supports all HTTP API endpoints.
 
     Attributes:
         users: operations with users
         projects: operations with projects
         backends: operations with backends
+        fleets: operations with fleets
         runs: operations with runs
+        metrics: operations with metrics
         logs: operations with logs
         gateways: operations with gateways
-        pools: operations with pools
+        volumes: operations with volumes
+        files: operations with files
     """
 
     def __init__(self, base_url: str, token: str):
         """
         Args:
-            base_url: API endpoints prefix, e.g. `http://127.0.0.1:3000/`
-            token: API token
+            base_url: The API endpoints prefix, e.g. `http://127.0.0.1:3000/`.
+            token: The API token.
         """
         self._base_url = base_url.rstrip("/")
         self._token = token
@@ -72,6 +81,10 @@ class APIClient:
     def backends(self) -> BackendsAPIClient:
         return BackendsAPIClient(self._request)
 
+    @property
+    def fleets(self) -> FleetsAPIClient:
+        return FleetsAPIClient(self._request)
+
     @property
     def repos(self) -> ReposAPIClient:
         return ReposAPIClient(self._request)
@@ -96,18 +109,14 @@ class APIClient:
     def gateways(self) -> GatewaysAPIClient:
         return GatewaysAPIClient(self._request)
 
-    @property
-    def pool(self) -> PoolAPIClient:
-        return PoolAPIClient(self._request)
-
-    @property
-    def fleets(self) -> FleetsAPIClient:
-        return FleetsAPIClient(self._request)
-
     @property
     def volumes(self) -> VolumesAPIClient:
         return VolumesAPIClient(self._request)
 
+    @property
+    def files(self) -> FilesAPIClient:
+        return FilesAPIClient(self._request)
+
     def _request(
         self,
         path: str,
@@ -133,9 +142,16 @@ class APIClient:
         else:
             raise ClientError(f"Failed to connect to dstack server {self._base_url}")
 
+        if 400 <= resp.status_code < 600:
+            logger.debug(
+                "Error requesting %s. Status: %s. Headers: %s. Body: %s",
+                resp.request.url,
+                resp.status_code,
+                resp.headers,
+                resp.content,
+            )
+
         if raise_for_status:
-            if resp.status_code == 500:
-                raise ClientError("Unexpected dstack server error")
             if resp.status_code == 400:  # raise ServerClientError
                 detail: List[Dict] = resp.json()["detail"]
                 if len(detail) == 1 and detail[0]["code"] in _server_client_errors:
@@ -145,7 +161,20 @@ class APIClient:
             if resp.status_code == 422:
                 formatted_error = pprint.pformat(resp.json())
                 raise ClientError(f"Server validation error: \n{formatted_error}")
-            resp.raise_for_status()
+            if resp.status_code == 403:
+                raise ClientError(
+                    f"Access to {resp.request.url} is denied. Please check your access token"
+                )
+            if resp.status_code == 404:
+                raise URLNotFoundError(f"Status code 404 when requesting {resp.request.url}")
+            if resp.status_code == 405:
+                raise MethodNotAllowedError(f"Status code 405 when requesting {resp.request.url}")
+            if 400 <= resp.status_code < 600:
+                raise ClientError(
+                    f"Unexpected error: status code {resp.status_code}"
+                    f" when requesting {resp.request.url}."
+                    " Check the server logs for backend issues, and the CLI logs at (~/.dstack/logs/cli/latest.log) local CLI output"
+                )
         return resp
 
 
