@@ -2,6 +2,7 @@ from typing import List
 
 from pydantic import parse_obj_as
 
+from dstack._internal.core.compatibility.volumes import get_create_volume_excludes
 from dstack._internal.core.models.volumes import Volume, VolumeConfiguration
 from dstack._internal.server.schemas.volumes import (
     CreateVolumeRequest,
@@ -27,7 +28,10 @@ class VolumesAPIClient(APIClientGroup):
         configuration: VolumeConfiguration,
     ) -> Volume:
         body = CreateVolumeRequest(configuration=configuration)
-        resp = self._request(f"/api/project/{project_name}/volumes/create", body=body.json())
+        resp = self._request(
+            f"/api/project/{project_name}/volumes/create",
+            body=body.json(exclude=get_create_volume_excludes(configuration)),
+        )
         return parse_obj_as(Volume.__response__, resp.json())
 
     def delete(self, project_name: str, names: List[str]) -> None:
