@@ -1,7 +1,7 @@
 import json
 from datetime import datetime, timezone
 from unittest.mock import Mock, patch
-from uuid import UUID
+from uuid import UUID, uuid4
 
 import pytest
 from freezegun import freeze_time
@@ -10,7 +10,12 @@ from sqlalchemy import select
 from sqlalchemy.ext.asyncio import AsyncSession
 
 from dstack._internal.core.models.backends.base import BackendType
-from dstack._internal.core.models.fleets import FleetConfiguration, FleetStatus, SSHParams
+from dstack._internal.core.models.fleets import (
+    FleetConfiguration,
+    FleetStatus,
+    InstanceGroupPlacement,
+    SSHParams,
+)
 from dstack._internal.core.models.instances import (
     InstanceAvailability,
     InstanceOfferWithAvailability,
@@ -21,13 +26,13 @@ from dstack._internal.core.models.instances import (
 )
 from dstack._internal.core.models.users import GlobalRole, ProjectRole
 from dstack._internal.server.models import FleetModel, InstanceModel
+from dstack._internal.server.services.fleets import fleet_model_to_fleet
 from dstack._internal.server.services.permissions import DefaultPermissions
 from dstack._internal.server.services.projects import add_project_member
 from dstack._internal.server.testing.common import (
     create_fleet,
     create_instance,
     create_job,
-    create_pool,
     create_project,
     create_repo,
     create_run,
@@ -36,9 +41,15 @@ from dstack._internal.server.testing.common import (
     get_auth_headers,
     get_fleet_configuration,
     get_fleet_spec,
+    get_instance_offer_with_availability,
+    get_job_provisioning_data,
     get_private_key_string,
+    get_remote_connection_info,
+    get_ssh_fleet_configuration,
 )
 
+pytestmark = pytest.mark.usefixtures("image_config_mock")
+
 
 class TestListFleets:
     @pytest.mark.asyncio
@@ -181,7 +192,10 @@ class TestGetFleet:
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
-    async def test_returns_fleet(self, test_db, session: AsyncSession, client: AsyncClient):
+    @pytest.mark.parametrize("deleted", [False, True])
+    async def test_returns_fleet_by_id(
+        self, test_db, session: AsyncSession, client: AsyncClient, deleted: bool
+    ):
         user = await create_user(session, global_role=GlobalRole.USER)
         project = await create_project(session)
         await add_project_member(
@@ -191,11 +205,12 @@ class TestGetFleet:
             session=session,
             project=project,
             created_at=datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
+            deleted=deleted,
         )
         response = await client.post(
             f"/api/project/{project.name}/fleets/get",
             headers=get_auth_headers(user.token),
-            json={"name": fleet.name},
+            json={"id": str(fleet.id)},
         )
         assert response.status_code == 200
         assert response.json() == {
@@ -211,7 +226,67 @@ class TestGetFleet:
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
-    async def test_returns_400_if_fleet_does_not_exist(
+    async def test_returns_not_deleted_fleet_by_name(
+        self, test_db, session: AsyncSession, client: AsyncClient
+    ):
+        user = await create_user(session, global_role=GlobalRole.USER)
+        project = await create_project(session)
+        await add_project_member(
+            session=session, project=project, user=user, project_role=ProjectRole.USER
+        )
+        active_fleet = await create_fleet(
+            session=session,
+            project=project,
+            created_at=datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc),
+            fleet_id=uuid4(),
+        )
+        deleted_fleet = await create_fleet(
+            session=session,
+            project=project,
+            created_at=datetime(2023, 1, 2, 3, 5, tzinfo=timezone.utc),
+            fleet_id=uuid4(),
+            deleted=True,
+        )
+        assert active_fleet.name == deleted_fleet.name
+        assert active_fleet.id != deleted_fleet.id
+        response = await client.post(
+            f"/api/project/{project.name}/fleets/get",
+            headers=get_auth_headers(user.token),
+            json={"name": active_fleet.name},
+        )
+        assert response.status_code == 200
+        assert response.json() == {
+            "id": str(active_fleet.id),
+            "name": active_fleet.name,
+            "project_name": project.name,
+            "spec": json.loads(active_fleet.spec),
+            "created_at": "2023-01-02T03:04:00+00:00",
+            "status": active_fleet.status.value,
+            "status_message": None,
+            "instances": [],
+        }
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
+    async def test_not_returns_by_name_if_fleet_deleted(
+        self, test_db, session: AsyncSession, client: AsyncClient
+    ):
+        user = await create_user(session, global_role=GlobalRole.USER)
+        project = await create_project(session)
+        await add_project_member(
+            session=session, project=project, user=user, project_role=ProjectRole.USER
+        )
+        fleet = await create_fleet(session=session, project=project, deleted=True)
+        response = await client.post(
+            f"/api/project/{project.name}/fleets/get",
+            headers=get_auth_headers(user.token),
+            json={"name": fleet.name},
+        )
+        assert response.status_code == 400
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
+    async def test_not_returns_by_name_if_fleet_does_not_exist(
         self, test_db, session: AsyncSession, client: AsyncClient
     ):
         user = await create_user(session, global_role=GlobalRole.USER)
@@ -227,13 +302,13 @@ class TestGetFleet:
         assert response.status_code == 400
 
 
-class TestCreateFleet:
+class TestApplyFleetPlan:
     @pytest.mark.asyncio
     @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
     async def test_returns_40x_if_not_authenticated(
         self, test_db, session: AsyncSession, client: AsyncClient
     ):
-        response = await client.post("/api/project/main/fleets/create")
+        response = await client.post("/api/project/main/fleets/apply")
         assert response.status_code == 403
 
     @pytest.mark.asyncio
@@ -249,9 +324,9 @@ class TestCreateFleet:
         with patch("uuid.uuid4") as m:
             m.return_value = UUID("1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e")
             response = await client.post(
-                f"/api/project/{project.name}/fleets/create",
+                f"/api/project/{project.name}/fleets/apply",
                 headers=get_auth_headers(user.token),
-                json={"spec": spec.dict()},
+                json={"plan": {"spec": spec.dict()}, "force": False},
             )
         assert response.status_code == 200
         assert response.json() == {
@@ -274,31 +349,39 @@ class TestCreateFleet:
                     },
                     "backends": None,
                     "regions": None,
+                    "availability_zones": None,
                     "instance_types": None,
                     "spot_policy": None,
                     "retry": None,
                     "max_price": None,
-                    "termination_policy": None,
-                    "termination_idle_time": None,
+                    "idle_duration": None,
                     "type": "fleet",
                     "name": "test-fleet",
+                    "reservation": None,
+                    "blocks": 1,
+                    "tags": None,
                 },
                 "profile": {
                     "backends": None,
                     "regions": None,
+                    "availability_zones": None,
                     "instance_types": None,
                     "spot_policy": None,
                     "retry": None,
-                    "retry_policy": None,
                     "max_duration": None,
+                    "stop_duration": None,
                     "max_price": None,
-                    "pool_name": None,
-                    "instance_name": None,
                     "creation_policy": None,
-                    "termination_policy": None,
-                    "termination_idle_time": None,
+                    "idle_duration": None,
+                    "utilization_policy": None,
+                    "startup_order": None,
+                    "stop_criteria": None,
+                    "schedule": None,
                     "name": "",
                     "default": False,
+                    "reservation": None,
+                    "fleets": None,
+                    "tags": None,
                 },
                 "autocreated": False,
             },
@@ -310,25 +393,30 @@ class TestCreateFleet:
                     "id": "1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e",
                     "project_name": project.name,
                     "name": f"{spec.configuration.name}-0",
+                    "fleet_id": "1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e",
+                    "fleet_name": spec.configuration.name,
                     "instance_num": 0,
                     "job_name": None,
                     "hostname": None,
                     "status": "pending",
                     "unreachable": False,
+                    "health_status": "healthy",
                     "termination_reason": None,
                     "created": "2023-01-02T03:04:00+00:00",
-                    "pool_name": None,
                     "backend": None,
                     "region": None,
+                    "availability_zone": None,
                     "instance_type": None,
                     "price": None,
+                    "total_blocks": 1,
+                    "busy_blocks": 0,
                 }
             ],
         }
         res = await session.execute(select(FleetModel))
         assert res.scalar_one()
         res = await session.execute(select(InstanceModel))
-        assert res.scalar_one()
+        assert res.unique().scalar_one()
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
@@ -339,23 +427,20 @@ class TestCreateFleet:
         await add_project_member(
             session=session, project=project, user=user, project_role=ProjectRole.USER
         )
-        spec = get_fleet_spec(
-            conf=FleetConfiguration(
-                name="test-ssh-fleet",
-                ssh_config=SSHParams(
-                    user="ubuntu",
-                    ssh_key=SSHKey(public="", private=get_private_key_string()),
-                    hosts=["1.1.1.1"],
-                    network=None,
-                ),
-            )
+        conf = get_ssh_fleet_configuration(
+            name="test-ssh-fleet",
+            user="ubuntu",
+            ssh_key=SSHKey(public="", private=get_private_key_string()),
+            hosts=["1.1.1.1"],
+            network=None,
         )
+        spec = get_fleet_spec(conf=conf)
         with patch("uuid.uuid4") as m:
             m.return_value = UUID("1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e")
             response = await client.post(
-                f"/api/project/{project.name}/fleets/create",
+                f"/api/project/{project.name}/fleets/apply",
                 headers=get_auth_headers(user.token),
-                json={"spec": spec.dict()},
+                json={"plan": {"spec": spec.dict()}, "force": False},
             )
         assert response.status_code == 200, response.json()
         assert response.json() == {
@@ -371,6 +456,7 @@ class TestCreateFleet:
                         "port": None,
                         "identity_file": None,
                         "ssh_key": None,  # should not return ssh_key
+                        "proxy_jump": None,
                         "hosts": ["1.1.1.1"],
                         "network": None,
                     },
@@ -385,31 +471,39 @@ class TestCreateFleet:
                     },
                     "backends": None,
                     "regions": None,
+                    "availability_zones": None,
                     "instance_types": None,
                     "spot_policy": None,
                     "retry": None,
                     "max_price": None,
-                    "termination_policy": None,
-                    "termination_idle_time": None,
+                    "idle_duration": None,
                     "type": "fleet",
                     "name": spec.configuration.name,
+                    "reservation": None,
+                    "blocks": 1,
+                    "tags": None,
                 },
                 "profile": {
                     "backends": None,
                     "regions": None,
+                    "availability_zones": None,
                     "instance_types": None,
                     "spot_policy": None,
                     "retry": None,
-                    "retry_policy": None,
                     "max_duration": None,
+                    "stop_duration": None,
                     "max_price": None,
-                    "pool_name": None,
-                    "instance_name": None,
                     "creation_policy": None,
-                    "termination_policy": None,
-                    "termination_idle_time": None,
+                    "idle_duration": None,
+                    "utilization_policy": None,
+                    "startup_order": None,
+                    "stop_criteria": None,
+                    "schedule": None,
                     "name": "",
                     "default": False,
+                    "reservation": None,
+                    "fleets": None,
+                    "tags": None,
                 },
                 "autocreated": False,
             },
@@ -424,32 +518,247 @@ class TestCreateFleet:
                     "instance_type": {
                         "name": "ssh",
                         "resources": {
+                            "cpu_arch": None,
                             "cpus": 2,
                             "memory_mib": 8,
                             "gpus": [],
                             "spot": False,
                             "disk": {"size_mib": 102400},
-                            "description": "",
+                            "description": "cpu=2 mem=0GB disk=100GB",
                         },
                     },
                     "name": f"{spec.configuration.name}-0",
+                    "fleet_id": "1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e",
+                    "fleet_name": spec.configuration.name,
                     "instance_num": 0,
-                    "pool_name": None,
                     "job_name": None,
                     "hostname": "1.1.1.1",
                     "status": "pending",
                     "unreachable": False,
+                    "health_status": "healthy",
                     "termination_reason": None,
                     "created": "2023-01-02T03:04:00+00:00",
                     "region": "remote",
+                    "availability_zone": None,
                     "price": 0.0,
+                    "total_blocks": 1,
+                    "busy_blocks": 0,
                 }
             ],
         }
         res = await session.execute(select(FleetModel))
         assert res.scalar_one()
         res = await session.execute(select(InstanceModel))
-        instance = res.scalar_one()
+        instance = res.unique().scalar_one()
+        assert instance.remote_connection_info is not None
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
+    @freeze_time(datetime(2023, 1, 2, 3, 4, tzinfo=timezone.utc), real_asyncio=True)
+    async def test_updates_ssh_fleet(self, test_db, session: AsyncSession, client: AsyncClient):
+        user = await create_user(session, global_role=GlobalRole.USER)
+        project = await create_project(session)
+        await add_project_member(
+            session=session, project=project, user=user, project_role=ProjectRole.USER
+        )
+        current_conf = get_ssh_fleet_configuration(
+            name="test-ssh-fleet",
+            user="ubuntu",
+            ssh_key=SSHKey(public="", private=get_private_key_string()),
+            hosts=["10.0.0.100"],
+            network=None,
+        )
+        current_spec = get_fleet_spec(conf=current_conf)
+        spec = current_spec.copy(deep=True)
+        # 10.0.0.100 removed, 10.0.0.101 added
+        spec.configuration.ssh_config.hosts = ["10.0.0.101"]
+
+        fleet = await create_fleet(session=session, project=project, spec=current_spec)
+        instance_type = InstanceType(
+            name="ssh",
+            resources=Resources(cpus=2, memory_mib=8, gpus=[], spot=False),
+        )
+        instance = await create_instance(
+            session=session,
+            project=project,
+            fleet=fleet,
+            backend=BackendType.REMOTE,
+            name="test-ssh-fleet-0",
+            region="remote",
+            price=0.0,
+            status=InstanceStatus.IDLE,
+            offer=get_instance_offer_with_availability(
+                backend=BackendType.REMOTE,
+                region="remote",
+                price=0.0,
+            ),
+            job_provisioning_data=get_job_provisioning_data(
+                instance_type=instance_type,
+                hostname="10.0.0.100",
+            ),
+            remote_connection_info=get_remote_connection_info(host="10.0.0.100"),
+        )
+
+        with patch("uuid.uuid4") as m:
+            m.return_value = UUID("1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e")
+            response = await client.post(
+                f"/api/project/{project.name}/fleets/apply",
+                headers=get_auth_headers(user.token),
+                json={
+                    "plan": {
+                        "spec": spec.dict(),
+                        "current_resource": _fleet_model_to_json_dict(fleet),
+                    },
+                    "force": False,
+                },
+            )
+
+        assert response.status_code == 200, response.json()
+        assert response.json() == {
+            "id": str(fleet.id),
+            "name": spec.configuration.name,
+            "project_name": project.name,
+            "spec": {
+                "configuration_path": spec.configuration_path,
+                "configuration": {
+                    "env": {},
+                    "ssh_config": {
+                        "user": "ubuntu",
+                        "port": None,
+                        "identity_file": None,
+                        "ssh_key": None,  # should not return ssh_key
+                        "proxy_jump": None,
+                        "hosts": ["10.0.0.101"],
+                        "network": None,
+                    },
+                    "nodes": None,
+                    "placement": None,
+                    "resources": {
+                        "cpu": {"min": 2, "max": None},
+                        "memory": {"min": 8.0, "max": None},
+                        "shm_size": None,
+                        "gpu": None,
+                        "disk": {"size": {"min": 100.0, "max": None}},
+                    },
+                    "backends": None,
+                    "regions": None,
+                    "availability_zones": None,
+                    "instance_types": None,
+                    "spot_policy": None,
+                    "retry": None,
+                    "max_price": None,
+                    "idle_duration": None,
+                    "type": "fleet",
+                    "name": spec.configuration.name,
+                    "reservation": None,
+                    "blocks": 1,
+                    "tags": None,
+                },
+                "profile": {
+                    "backends": None,
+                    "regions": None,
+                    "availability_zones": None,
+                    "instance_types": None,
+                    "spot_policy": None,
+                    "retry": None,
+                    "max_duration": None,
+                    "stop_duration": None,
+                    "max_price": None,
+                    "creation_policy": None,
+                    "idle_duration": None,
+                    "utilization_policy": None,
+                    "startup_order": None,
+                    "stop_criteria": None,
+                    "schedule": None,
+                    "name": "",
+                    "default": False,
+                    "reservation": None,
+                    "fleets": None,
+                    "tags": None,
+                },
+                "autocreated": False,
+            },
+            "created_at": "2023-01-02T03:04:00+00:00",
+            "status": "active",
+            "status_message": None,
+            "instances": [
+                {
+                    "id": str(instance.id),
+                    "project_name": project.name,
+                    "backend": "remote",
+                    "instance_type": {
+                        "name": "ssh",
+                        "resources": {
+                            "cpu_arch": None,
+                            "cpus": 2,
+                            "memory_mib": 8,
+                            "gpus": [],
+                            "spot": False,
+                            "disk": {"size_mib": 102400},
+                            "description": "cpu=2 mem=0GB disk=100GB",
+                        },
+                    },
+                    "name": "test-ssh-fleet-0",
+                    "fleet_id": str(fleet.id),
+                    "fleet_name": "test-ssh-fleet",
+                    "instance_num": 0,
+                    "job_name": None,
+                    "hostname": "10.0.0.100",
+                    "status": "terminating",
+                    "unreachable": False,
+                    "health_status": "healthy",
+                    "termination_reason": None,
+                    "created": "2023-01-02T03:04:00+00:00",
+                    "region": "remote",
+                    "availability_zone": None,
+                    "price": 0.0,
+                    "total_blocks": 1,
+                    "busy_blocks": 0,
+                },
+                {
+                    "id": "1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e",
+                    "project_name": project.name,
+                    "backend": "remote",
+                    "instance_type": {
+                        "name": "ssh",
+                        "resources": {
+                            "cpu_arch": None,
+                            "cpus": 2,
+                            "memory_mib": 8,
+                            "gpus": [],
+                            "spot": False,
+                            "disk": {"size_mib": 102400},
+                            "description": "cpu=2 mem=0GB disk=100GB",
+                        },
+                    },
+                    "name": "test-ssh-fleet-1",
+                    "fleet_id": str(fleet.id),
+                    "fleet_name": "test-ssh-fleet",
+                    "instance_num": 1,
+                    "job_name": None,
+                    "hostname": "10.0.0.101",
+                    "status": "pending",
+                    "unreachable": False,
+                    "health_status": "healthy",
+                    "termination_reason": None,
+                    "created": "2023-01-02T03:04:00+00:00",
+                    "region": "remote",
+                    "availability_zone": None,
+                    "price": 0.0,
+                    "total_blocks": 1,
+                    "busy_blocks": 0,
+                },
+            ],
+        }
+        res = await session.execute(select(FleetModel))
+        assert res.scalar_one()
+        await session.refresh(instance)
+        assert instance.status == InstanceStatus.TERMINATING
+        res = await session.execute(
+            select(InstanceModel).where(InstanceModel.id == "1b0e1b45-2f8c-4ab6-8010-a0d1a3e44e0e")
+        )
+        instance = res.unique().scalar_one()
+        assert instance.status == InstanceStatus.PENDING
         assert instance.remote_connection_info is not None
 
     @pytest.mark.asyncio
@@ -475,9 +784,9 @@ class TestCreateFleet:
             )
         )
         response = await client.post(
-            f"/api/project/{project.name}/fleets/create",
+            f"/api/project/{project.name}/fleets/apply",
             headers=get_auth_headers(user.token),
-            json={"spec": spec.dict()},
+            json={"plan": {"spec": spec.dict()}, "force": False},
         )
         assert response.status_code == 400
 
@@ -506,9 +815,9 @@ class TestCreateFleet:
             DefaultPermissions(allow_non_admins_manage_ssh_fleets=False)
         ):
             response = await client.post(
-                f"/api/project/{project.name}/fleets/create",
+                f"/api/project/{project.name}/fleets/apply",
                 headers=get_auth_headers(user.token),
-                json={"spec": spec.dict()},
+                json={"plan": {"spec": spec.dict()}, "force": False},
             )
         assert response.status_code in [401, 403]
 
@@ -532,12 +841,10 @@ class TestDeleteFleets:
         await add_project_member(
             session=session, project=project, user=user, project_role=ProjectRole.USER
         )
-        pool = await create_pool(session=session, project=project)
         fleet = await create_fleet(session=session, project=project)
         instance = await create_instance(
             session=session,
             project=project,
-            pool=pool,
         )
         fleet.instances.append(instance)
         await session.commit()
@@ -559,7 +866,6 @@ class TestDeleteFleets:
     ):
         user = await create_user(session, global_role=GlobalRole.USER)
         project = await create_project(session)
-        pool = await create_pool(session=session, project=project)
         await add_project_member(
             session=session, project=project, user=user, project_role=ProjectRole.USER
         )
@@ -581,7 +887,6 @@ class TestDeleteFleets:
         instance = await create_instance(
             session=session,
             project=project,
-            pool=pool,
             status=InstanceStatus.BUSY,
             job=job,
         )
@@ -649,18 +954,15 @@ class TestDeleteFleetInstances:
         await add_project_member(
             session=session, project=project, user=user, project_role=ProjectRole.USER
         )
-        pool = await create_pool(session=session, project=project)
         fleet = await create_fleet(session=session, project=project)
         instance1 = await create_instance(
             session=session,
             project=project,
-            pool=pool,
             instance_num=1,
         )
         instance2 = await create_instance(
             session=session,
             project=project,
-            pool=pool,
             instance_num=2,
         )
         fleet.instances.append(instance1)
@@ -690,7 +992,6 @@ class TestDeleteFleetInstances:
         await add_project_member(
             session=session, project=project, user=user, project_role=ProjectRole.USER
         )
-        pool = await create_pool(session=session, project=project)
         fleet = await create_fleet(session=session, project=project)
         repo = await create_repo(
             session=session,
@@ -709,7 +1010,6 @@ class TestDeleteFleetInstances:
         instance = await create_instance(
             session=session,
             project=project,
-            pool=pool,
             instance_num=1,
             status=InstanceStatus.BUSY,
             job=job,
@@ -740,7 +1040,9 @@ class TestGetPlan:
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
-    async def test_returns_plan(self, test_db, session: AsyncSession, client: AsyncClient):
+    async def test_returns_create_plan_for_new_fleet(
+        self, test_db, session: AsyncSession, client: AsyncClient
+    ):
         user = await create_user(session=session, global_role=GlobalRole.USER)
         project = await create_project(session=session, owner=user)
         await add_project_member(
@@ -763,21 +1065,103 @@ class TestGetPlan:
             backend_mock = Mock()
             m.return_value = [backend_mock]
             backend_mock.TYPE = BackendType.AWS
-            backend_mock.compute.return_value.get_offers.return_value = offers
+            backend_mock.compute.return_value.get_offers_cached.return_value = offers
             response = await client.post(
                 f"/api/project/{project.name}/fleets/get_plan",
                 headers=get_auth_headers(user.token),
                 json={"spec": spec.dict()},
             )
-            backend_mock.compute.return_value.get_offers.assert_called_once()
+            backend_mock.compute.return_value.get_offers_cached.assert_called_once()
+
+        assert response.status_code == 200
+        assert response.json() == {
+            "project_name": project.name,
+            "user": user.name,
+            "spec": json.loads(spec.json()),
+            "effective_spec": json.loads(spec.json()),
+            "current_resource": None,
+            "offers": [json.loads(o.json()) for o in offers],
+            "total_offers": len(offers),
+            "max_offer_price": 1.0,
+            "action": "create",
+        }
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
+    async def test_returns_update_plan_for_existing_fleet(
+        self, test_db, session: AsyncSession, client: AsyncClient
+    ):
+        user = await create_user(session=session, global_role=GlobalRole.USER)
+        project = await create_project(session=session, owner=user)
+        await add_project_member(
+            session=session, project=project, user=user, project_role=ProjectRole.USER
+        )
+        conf = get_ssh_fleet_configuration(hosts=["10.0.0.100"])
+        spec = get_fleet_spec(conf=conf)
+        effective_spec = spec.copy(deep=True)
+        effective_spec.configuration.ssh_config.ssh_key = None
+        current_spec = spec.copy(deep=True)
+        # `hosts` can be updated in-place
+        current_spec.configuration.ssh_config.hosts = ["10.0.0.100", "10.0.0.101"]
+        fleet = await create_fleet(session=session, project=project, spec=current_spec)
+
+        response = await client.post(
+            f"/api/project/{project.name}/fleets/get_plan",
+            headers=get_auth_headers(user.token),
+            json={"spec": spec.dict()},
+        )
 
         assert response.status_code == 200
         assert response.json() == {
             "project_name": project.name,
             "user": user.name,
             "spec": spec.dict(),
-            "current_resource": None,
-            "offers": [json.loads(o.json()) for o in offers],
-            "total_offers": len(offers),
-            "max_offer_price": 1.0,
+            "effective_spec": effective_spec.dict(),
+            "current_resource": _fleet_model_to_json_dict(fleet),
+            "offers": [],
+            "total_offers": 0,
+            "max_offer_price": None,
+            "action": "update",
         }
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
+    async def test_returns_create_plan_for_existing_fleet(
+        self, test_db, session: AsyncSession, client: AsyncClient
+    ):
+        user = await create_user(session=session, global_role=GlobalRole.USER)
+        project = await create_project(session=session, owner=user)
+        await add_project_member(
+            session=session, project=project, user=user, project_role=ProjectRole.USER
+        )
+        conf = get_ssh_fleet_configuration(placement=InstanceGroupPlacement.ANY)
+        spec = get_fleet_spec(conf=conf)
+        effective_spec = spec.copy(deep=True)
+        effective_spec.configuration.ssh_config.ssh_key = None
+        current_spec = spec.copy(deep=True)
+        # `placement` cannot be updated in-place
+        current_spec.configuration.placement = InstanceGroupPlacement.CLUSTER
+        fleet = await create_fleet(session=session, project=project, spec=current_spec)
+
+        response = await client.post(
+            f"/api/project/{project.name}/fleets/get_plan",
+            headers=get_auth_headers(user.token),
+            json={"spec": spec.dict()},
+        )
+
+        assert response.status_code == 200
+        assert response.json() == {
+            "project_name": project.name,
+            "user": user.name,
+            "spec": spec.dict(),
+            "effective_spec": effective_spec.dict(),
+            "current_resource": _fleet_model_to_json_dict(fleet),
+            "offers": [],
+            "total_offers": 0,
+            "max_offer_price": None,
+            "action": "create",
+        }
+
+
+def _fleet_model_to_json_dict(fleet: FleetModel) -> dict:
+    return json.loads(fleet_model_to_fleet(fleet).json())
