@@ -7,12 +7,9 @@ from sqlalchemy.ext.asyncio import AsyncSession
 from dstack._internal.core.errors import DstackError
 from dstack._internal.core.models.backends.base import BackendType
 from dstack._internal.core.models.users import GlobalRole, ProjectRole
-from dstack._internal.server.services.gateways import (
-    gateway_model_to_gateway,
-    get_project_default_gateway,
-)
 from dstack._internal.server.services.projects import add_project_member
 from dstack._internal.server.testing.common import (
+    ComputeMockSpec,
     create_backend,
     create_gateway,
     create_gateway_compute,
@@ -77,6 +74,7 @@ class TestListAndGetGateways:
                     "default": False,
                     "public_ip": True,
                     "certificate": {"type": "lets-encrypt"},
+                    "tags": None,
                 },
             }
         ]
@@ -127,6 +125,7 @@ class TestListAndGetGateways:
                 "default": False,
                 "public_ip": True,
                 "certificate": {"type": "lets-encrypt"},
+                "tags": None,
             },
         }
 
@@ -174,7 +173,14 @@ class TestCreateGateway:
         backend = await create_backend(session, project.id, backend_type=BackendType.AWS)
         response = await client.post(
             f"/api/project/{project.name}/gateways/create",
-            json={"name": "test", "backend_type": "aws", "region": "us"},
+            json={
+                "configuration": {
+                    "type": "gateway",
+                    "name": "test",
+                    "backend": "aws",
+                    "region": "us",
+                },
+            },
             headers=get_auth_headers(user.token),
         )
         assert response.status_code == 200
@@ -199,6 +205,7 @@ class TestCreateGateway:
                 "default": True,
                 "public_ip": True,
                 "certificate": {"type": "lets-encrypt"},
+                "tags": None,
             },
         }
 
@@ -217,7 +224,14 @@ class TestCreateGateway:
             g.return_value = "random-name"
             response = await client.post(
                 f"/api/project/{project.name}/gateways/create",
-                json={"name": None, "backend_type": "aws", "region": "us"},
+                json={
+                    "configuration": {
+                        "type": "gateway",
+                        "name": None,
+                        "backend": "aws",
+                        "region": "us",
+                    },
+                },
                 headers=get_auth_headers(user.token),
             )
             g.assert_called_once()
@@ -243,6 +257,7 @@ class TestCreateGateway:
                 "default": True,
                 "public_ip": True,
                 "certificate": {"type": "lets-encrypt"},
+                "tags": None,
             },
         }
 
@@ -258,39 +273,20 @@ class TestCreateGateway:
         )
         response = await client.post(
             f"/api/project/{project.name}/gateways/create",
-            json={"name": "test", "backend_type": "aws", "region": "us"},
+            json={
+                "configuration": {
+                    "type": "gateway",
+                    "name": "test",
+                    "backend": "aws",
+                    "region": "us",
+                },
+            },
             headers=get_auth_headers(user.token),
         )
         assert response.status_code == 400
 
 
 class TestDefaultGateway:
-    @pytest.mark.asyncio
-    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
-    async def test_get_default_gateway(self, test_db, session: AsyncSession, client: AsyncClient):
-        project = await create_project(session)
-        backend = await create_backend(session, project.id)
-        gateway = await create_gateway(session, project.id, backend.id)
-        async with session.begin():
-            project.default_gateway_id = gateway.id
-            session.add(project)
-
-        res = await get_project_default_gateway(session, project)
-        assert res is not None
-        assert res.dict() == gateway_model_to_gateway(gateway).dict()
-
-    @pytest.mark.asyncio
-    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
-    async def test_default_gateway_is_missing(
-        self, test_db, session: AsyncSession, client: AsyncClient
-    ):
-        project = await create_project(session)
-        backend = await create_backend(session, project.id)
-        await create_gateway(session, project.id, backend.id)
-
-        res = await get_project_default_gateway(session, project)
-        assert res is None
-
     @pytest.mark.asyncio
     @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
     async def test_only_admin_can_set_default_gateway(
@@ -363,6 +359,7 @@ class TestDefaultGateway:
                 "default": True,
                 "public_ip": True,
                 "certificate": {"type": "lets-encrypt"},
+                "tags": None,
             },
         }
 
@@ -437,8 +434,10 @@ class TestDeleteGateway:
             "dstack._internal.server.services.gateways.get_project_backend_by_type_or_error"
         ) as m:
             aws = Mock()
+            aws.compute.return_value = Mock(spec=ComputeMockSpec)
             aws.compute.return_value.terminate_gateway.return_value = None  # success
             gcp = Mock()
+            gcp.compute.return_value = Mock(spec=ComputeMockSpec)
             gcp.compute.return_value.terminate_gateway.side_effect = DstackError()  # fail
 
             def get_backend(project, backend_type):
@@ -482,6 +481,7 @@ class TestDeleteGateway:
                     "default": False,
                     "public_ip": True,
                     "certificate": {"type": "lets-encrypt"},
+                    "tags": None,
                 },
             }
         ]
@@ -550,6 +550,7 @@ class TestUpdateGateway:
                 "default": False,
                 "public_ip": True,
                 "certificate": {"type": "lets-encrypt"},
+                "tags": None,
             },
         }
 
