@@ -3,7 +3,7 @@ from httpx import AsyncClient
 from sqlalchemy.ext.asyncio import AsyncSession
 
 from dstack._internal.core.models.users import GlobalRole, ProjectRole
-from dstack._internal.server.services.logs import FileLogStorage
+from dstack._internal.server.services.logs.filelog import FileLogStorage
 from dstack._internal.server.services.projects import add_project_member
 from dstack._internal.server.testing.common import create_project, create_user, get_auth_headers
 
@@ -62,19 +62,21 @@ class TestPollLogs:
                 {
                     "timestamp": "2023-10-06T10:01:53.234234+00:00",
                     "log_source": "stdout",
-                    "message": "Hello",
+                    "message": "SGVsbG8=",
                 },
                 {
                     "timestamp": "2023-10-06T10:01:53.234235+00:00",
                     "log_source": "stdout",
-                    "message": "World",
+                    "message": "V29ybGQ=",
                 },
                 {
                     "timestamp": "2023-10-06T10:01:53.234236+00:00",
                     "log_source": "stdout",
-                    "message": "!",
+                    "message": "IQ==",
                 },
-            ]
+            ],
+            "external_url": None,
+            "next_token": None,
         }
         response = await client.post(
             f"/api/project/{project.name}/logs/poll",
@@ -92,7 +94,9 @@ class TestPollLogs:
                 {
                     "timestamp": "2023-10-06T10:01:53.234236+00:00",
                     "log_source": "stdout",
-                    "message": "!",
+                    "message": "IQ==",
                 },
-            ]
+            ],
+            "external_url": None,
+            "next_token": None,
         }
