@@ -1,21 +1,25 @@
-from typing import List
+from typing import List, Union
 
 import pytest
 from pydantic import parse_obj_as
+from sqlalchemy import select
 from sqlalchemy.ext.asyncio import AsyncSession
+from sqlalchemy.orm import joinedload, selectinload
 
-from dstack._internal.core.errors import ServerClientError, ServerError
+from dstack._internal.core.errors import ServerClientError
 from dstack._internal.core.models.backends.base import BackendType
 from dstack._internal.core.models.configurations import ScalingSpec, ServiceConfiguration
 from dstack._internal.core.models.profiles import Profile
 from dstack._internal.core.models.resources import Range
 from dstack._internal.core.models.runs import JobStatus, JobTerminationReason, RunStatus
-from dstack._internal.core.models.volumes import VolumeMountPoint
-from dstack._internal.server.models import RunModel
-from dstack._internal.server.services.runs import check_can_attach_run_volumes, scale_run_replicas
+from dstack._internal.server.models import (
+    JobModel,
+    RunModel,
+)
+from dstack._internal.server.services.jobs import check_can_attach_job_volumes
+from dstack._internal.server.services.runs import scale_run_replicas
 from dstack._internal.server.testing.common import (
     create_job,
-    create_pool,
     create_project,
     create_repo,
     create_run,
@@ -24,12 +28,14 @@ from dstack._internal.server.testing.common import (
     get_volume,
 )
 
+pytestmark = pytest.mark.usefixtures("image_config_mock")
+
 
 async def make_run(
     session: AsyncSession,
     replicas_statuses: List[JobStatus],
     status: RunStatus = RunStatus.RUNNING,
-    replicas: str = 1,
+    replicas: Union[str, int] = 1,
 ) -> RunModel:
     project = await create_project(session=session)
     user = await create_user(session=session)
@@ -37,9 +43,6 @@ async def make_run(
         session=session,
         project_id=project.id,
     )
-    project.default_pool = await create_pool(
-        session=session, project=project, pool_name="default-pool"
-    )
     run_name = "test-run"
     profile = Profile(name="test-profile")
     run_spec = get_run_spec(
@@ -66,14 +69,21 @@ async def make_run(
         status=status,
     )
     for replica_num, job_status in enumerate(replicas_statuses):
-        await create_job(
+        job = await create_job(
             session=session,
             run=run,
             status=job_status,
             replica_num=replica_num,
         )
-    await session.refresh(run)
-    return run
+        run.jobs.append(job)
+    res = await session.execute(
+        select(RunModel)
+        .where(RunModel.id == run.id)
+        .options(joinedload(RunModel.project))
+        .options(selectinload(RunModel.jobs).joinedload(JobModel.probes))
+        .execution_options(populate_existing=True)
+    )
+    return res.unique().scalar_one()
 
 
 async def scale_wrapper(session: AsyncSession, run: RunModel, diff: int):
@@ -178,32 +188,6 @@ class TestScaleRunReplicas:
         assert run.jobs[1].status == JobStatus.TERMINATING
         assert run.jobs[1].termination_reason == JobTerminationReason.SCALED_DOWN
 
-    @pytest.mark.asyncio
-    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
-    async def test_no_downscale_below_limit(self, test_db, session: AsyncSession):
-        run = await make_run(
-            session,
-            [
-                JobStatus.RUNNING,
-            ],
-            replicas="1..2",
-        )
-        with pytest.raises(ServerError):
-            await scale_wrapper(session, run, -1)
-
-    @pytest.mark.asyncio
-    @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
-    async def test_no_upscale_above_limit(self, test_db, session: AsyncSession):
-        run = await make_run(
-            session,
-            [
-                JobStatus.RUNNING,
-            ],
-            replicas="0..1",
-        )
-        with pytest.raises(ServerError):
-            await scale_wrapper(session, run, 1)
-
     @pytest.mark.asyncio
     @pytest.mark.parametrize("test_db", ["sqlite", "postgres"], indirect=True)
     async def test_upscale_mixed(self, test_db, session: AsyncSession):
@@ -239,19 +223,7 @@ class TestCanAttachRunVolumes:
         vol22.configuration.backend = BackendType.AWS
         vol22.configuration.region = "eu-west-2"
         volumes = [[vol11, vol12], [vol21, vol22]]
-        run_spec = get_run_spec(
-            run_name="test_run",
-            repo_id="test_repo",
-            configuration=ServiceConfiguration(
-                port=80,
-                commands=[""],
-                volumes=[
-                    VolumeMountPoint(name=["vol11", "vol12"], path="/vol1"),
-                    VolumeMountPoint(name=["vol21", "vol22"], path="/vol2"),
-                ],
-            ),
-        )
-        check_can_attach_run_volumes(run_spec, volumes)
+        check_can_attach_job_volumes(volumes)
 
     @pytest.mark.asyncio
     async def test_cannot_attach_different_mount_points_with_different_backends_regions(self):
@@ -262,20 +234,8 @@ class TestCanAttachRunVolumes:
         vol2.configuration.backend = BackendType.AWS
         vol2.configuration.region = "eu-west-2"
         volumes = [[vol1], [vol2]]
-        run_spec = get_run_spec(
-            run_name="test_run",
-            repo_id="test_repo",
-            configuration=ServiceConfiguration(
-                port=80,
-                commands=[""],
-                volumes=[
-                    VolumeMountPoint(name=["vol1"], path="/vol1"),
-                    VolumeMountPoint(name=["vol2"], path="/vol2"),
-                ],
-            ),
-        )
         with pytest.raises(ServerClientError):
-            check_can_attach_run_volumes(run_spec, volumes)
+            check_can_attach_job_volumes(volumes)
 
     @pytest.mark.asyncio
     async def test_cannot_attach_same_volume_at_different_mount_points(self):
@@ -283,17 +243,5 @@ class TestCanAttachRunVolumes:
         vol1.configuration.backend = BackendType.AWS
         vol1.configuration.region = "eu-west-1"
         volumes = [[vol1], [vol1]]
-        run_spec = get_run_spec(
-            run_name="test_run",
-            repo_id="test_repo",
-            configuration=ServiceConfiguration(
-                port=80,
-                commands=[""],
-                volumes=[
-                    VolumeMountPoint(name=["vol1"], path="/vol1"),
-                    VolumeMountPoint(name=["vol1"], path="/vol2"),
-                ],
-            ),
-        )
         with pytest.raises(ServerClientError):
-            check_can_attach_run_volumes(run_spec, volumes)
+            check_can_attach_job_volumes(volumes)
